#include "GameObject.hpp"
#include "Scene.hpp"

#include <map>
#include <set>
#include <glm/glm.hpp>
#include <glm/gtx/transform.hpp> 
#include <glm/gtc/type_ptr.hpp> 
#include <glm/gtx/random.hpp>
#include <glm/gtc/matrix_transform.hpp>

int GameObject::OBJECTIDCOUNTER = 0;

GameObject::GameObject(void) { init(); }
GameObject::GameObject(GameObject::Types type, Sector *sector)
{
	this->mType = type;
	init();
	this->mSector = sector;
	if (type == GameObject::Types::PersonGood || type == GameObject::Types::PersonBad) { mRadiusBoundingSphere = 20; }
	else if (type == GameObject::Types::Tree) { mRadiusBoundingSphere = 100; }
	else if (type == GameObject::Types::HolyGrenade) { mRadiusBoundingSphere = 40; }
	else if (type == GameObject::Types::Moai) { mRadiusBoundingSphere = 40; }
	else if (type == GameObject::Types::Ground) { mRadiusBoundingSphere = 1000; }
}

GameObject::~GameObject(void)
{
	delete mPosition;
	delete mRotation;
	delete mScale;
	delete mRotationSpeed;
	delete mRotationMax;

}

void GameObject::init(void)
{
	this->mParent = NULL;
	this->mSector = NULL;
	this->mPosition = new glm::vec3(0.0f,0.0f,0.0f);
	this->mRotation = new glm::vec3(0.0f,0.0f,0.0f);
	this->mScale = new glm::vec3(1.0f,1.0f,1.0f);
	this->mDirection = new glm::vec3(0.0f,0.0f,-1.0f);
	//this->mPositionPixelSpace = new glm::vec3(0,0,0);
	this->mVisible = true;
	this->mState = this->Moving;
	this->mRotationSpeed = new glm::vec3(0.0f,0.0f,0.0f);
	this->mRotationMax = new glm::vec3(0.0f, 0.0f, 0.0f);
	this->mIgnoreTransformations = false;
	this->mHighlighted = false;
	this->mTimeDiffDyingTime = 0.0f;
	this->mObjectId = OBJECTIDCOUNTER;
	OBJECTIDCOUNTER++;	
	Scene::sGameObjects.insert(std::pair<int, GameObject*>(this->mObjectId, this));
}

glm::mat4 GameObject::getModelMatrix(void) const
{
	if (this->mIgnoreTransformations) { return glm::mat4(); }
	return glm::translate(*mPosition) * glm::rotate(mRotation->y,0.0f,1.0f,0.0f) * glm::rotate(mRotation->z,1.0f,0.0f,0.0f) * glm::rotate(mRotation->x,0.0f,0.0f,1.0f) * glm::scale(*mScale); 
}

glm::vec3 GameObject::getDirectionXZ(void) const
{
	return glm::vec3(glm::mat3(glm::rotate(mRotation->y,0.0f,1.0f,0.0f)) * glm::vec3(1,0,0));
}

void GameObject::setDirectionXZ(glm::vec3 dir)
{
	float dot = glm::dot(glm::normalize(dir), glm::vec3(1,0,0));
	if (dir.z < 0) 
	{ this->mRotation->y = glm::degrees(glm::acos(dot)); }
	else 
	{ this->mRotation->y = 360.0f - glm::degrees(glm::acos(dot)); }
}

glm::vec3 GameObject::getPositionXZ(void) const
{
	return glm::vec3(mPosition->x,0.0f,mPosition->z);
}

