#include "PickingHelper.hpp"

PickingHelper* PickingHelper::singletonPointer = 0;

PickingHelper* PickingHelper::getInstance() {
    if(singletonPointer==0)
		singletonPointer=new PickingHelper();
    return singletonPointer;
}

PickingHelper::PickingHelper()
{
	m_useLinearScaling = true;
	m_useNaiveSearch = false;
	m_showBubbleCursor = false;
	m_showPickbufferRectangle = false;
	m_mouse_min_scale = 0.5;
	m_mouse_max_scale = 1.0;
	m_mouse_scaling_exponent = 6.0;
	m_mouse_cutoff_distance = 200;
	m_pickingbuffer_window_width = 601;
}

void PickingHelper::checkNN(const GLint* pickbuffer_window, const int x_size, const int y_size, const int x_center, const int y_center, int& id, double& distance)
{
	if(m_useNaiveSearch) {
		checkNN_naive(pickbuffer_window, x_size, y_size, x_center, y_center, id, distance);
	}
	else 
	{
		checkNN_sophisticated(pickbuffer_window, x_size, y_size, x_center, y_center, id, distance);
	}
}

void PickingHelper::checkNN_naive(const GLint* pickbuffer_window, const int x_size, const int y_size, const int x_center, const int y_center, int& id, double& distance) 
{
	int center_index = x_center + x_size * y_center;
	distance = DBL_MAX;
	id = 0;
	//check if mousecursor hit an object
	if (pickbuffer_window[center_index] > 0)
	{
		id = pickbuffer_window[center_index];
		distance = 0.0;
		return;
	}

	for(int i_x = 0; i_x < x_size; i_x++)
	{
		for(int i_y = 0; i_y < y_size; i_y++)
		{
			int current_id = pickbuffer_window[i_x + i_y * x_size];
			if(current_id > 0)
			{
				int x_offset = abs(x_center - i_x);
				int y_offset = abs(y_center - i_y);
				double current_distance = sqrt(x_offset*x_offset + y_offset*y_offset);
				if(current_distance < distance) {
					distance = current_distance;
					id = current_id;
				}
			}
		}
	}
}

void PickingHelper::checkNN_sophisticated(const GLint* pickbuffer_window, const int x_size, const int y_size, const int x_center, const int y_center, int& id, double& distance) 
{
	int center_index = x_center + x_size * y_center;
	distance = DBL_MAX;
	id = 0;
	//check if mousecursor hit an object
	if (pickbuffer_window[center_index] > 0)
	{
		id = pickbuffer_window[center_index];
		distance = 0.0;
		return;
	}
	//else search neighborhood
	bool neighbor_found = false;
	int search_radius = 1;
	
	//BOUNDS
	int x_max_offset_left = x_center;
	int x_max_offset_right = x_size - x_center - 1;
	int y_max_offset_up = y_center;
	int y_max_offset_down = y_size - y_center - 1;
	bool reached_left_bound = false;
	bool reached_right_bound = false;
	bool reached_upper_bound = false;
	bool reached_lower_bound = false;
	int search_radius_max = glm::max(glm::max(x_max_offset_left, x_max_offset_right), glm::max(y_max_offset_up, y_max_offset_down));
	
	while (!neighbor_found && search_radius <= search_radius_max)
	{
		//window bounds
		int x_min = glm::min(x_max_offset_left, search_radius);
		int x_max = glm::min(x_max_offset_right, search_radius);
		int y_min = glm::min(y_max_offset_up, search_radius);
		int y_max = glm::min(y_max_offset_down, search_radius);

		if(!(reached_upper_bound && reached_lower_bound) )
		{
			//search North / South
			for (int x_offset = -x_min; x_offset <= x_max ; x_offset++)
			{
				if(!reached_upper_bound)
				{
					int y_offset = -y_min;
					// search northern border
					int current_id = pickbuffer_window[x_center + x_offset + x_size * (y_center + y_offset)];
					if (current_id > 0) {
						//found one
						double current_distance = sqrt(x_offset*x_offset + y_offset*y_offset);
						if (current_distance < distance) 
						{
							distance = current_distance;
							id = current_id;
							neighbor_found = true;
						}
					}
				}
				// search southern border
				if(!reached_lower_bound)
				{
					int y_offset = y_max;
					int index = x_center + x_offset + x_size * (y_center + y_offset);
					int current_id = pickbuffer_window[index];
					if (current_id > 0) {
						//found one
						double current_distance = sqrt(x_offset*x_offset + y_offset*y_offset);
						if (current_distance < distance) 
						{
							distance = current_distance;
							id = current_id;
							neighbor_found = true;
						}
					}
				}
			}
		}
		if(!(reached_left_bound && reached_right_bound) )
		{
			//search Left / Right
			for (int y_offset = -y_min; y_offset <= y_max ; y_offset++)
			{
				if(!reached_left_bound)
				{
					int x_offset = -x_min;
					// search northern border
					int current_id = pickbuffer_window[x_center + x_offset + x_size * (y_center + y_offset)];
					if (current_id > 0) {
						//found one
						double current_distance = sqrt(x_offset*x_offset + y_offset*y_offset);
						if (current_distance < distance) 
						{
							distance = current_distance;
							id = current_id;
							neighbor_found = true;
						}
					}
				}
				if(!reached_right_bound)
				{
					// search southern border
					int x_offset = x_max;
					int current_id = pickbuffer_window[x_center + x_offset + x_size * (y_center + y_offset)];
					if (current_id > 0) {
						//found one
						double current_distance = sqrt(x_offset*x_offset + y_offset*y_offset);
						if (current_distance < distance) 
						{
							distance = current_distance;
							id = current_id;
							neighbor_found = true;
						}
					}
				}
			}
		}
		reached_left_bound = (x_min == x_max_offset_left);
		reached_right_bound = (x_max == x_max_offset_right);
		reached_upper_bound = (y_min == y_max_offset_up);
		reached_lower_bound = (y_max == y_max_offset_down);
		search_radius++;
	}
}

double PickingHelper::mouse_scale_linear(double distance)
{

	if (distance <= 0.0)
	{
		return m_mouse_min_scale;
	} 
	else 
	{
		if (distance > m_mouse_cutoff_distance)
		{
			return m_mouse_max_scale;
		}
		else
		{
			return distance/m_mouse_cutoff_distance * (m_mouse_max_scale - m_mouse_min_scale) + m_mouse_min_scale;
		} 
	}
}

double PickingHelper::mouse_scale_inverse(double distance)
{
	double normalized_distance = glm::min(distance, m_mouse_cutoff_distance) / m_mouse_cutoff_distance;
	return  (m_mouse_min_scale - m_mouse_max_scale) / pow(normalized_distance + 1.0, m_mouse_scaling_exponent) + m_mouse_max_scale;
}

double PickingHelper::scale_by_distance(double distance) {
	if(m_useLinearScaling) {
		return mouse_scale_linear(distance);
	} else {
		return mouse_scale_inverse(distance);
	}
}

double PickingHelper::springInput(double lastValue, double targetValue, double deltaTime, double springConstant = 6.0) {
	double factor = exp(-deltaTime * springConstant);
	return targetValue + (lastValue - targetValue) * factor;
};
