#include "PickingHelper.hpp"
#include "Scene.hpp"
#include "Model.hpp"
#include "Sector.hpp"
#include "common.hpp"
#include "utility.hpp"
#include "SoundInterface.hpp"
#include "Plane.hpp"


std::map<int,GameObject*> Scene::sGameObjects = std::map<int, GameObject*>();

static bool camchanged = false;
Scene::Scene(void)
{
	sGameObjects.clear();
	mScreenWidth = SCREEN_WIDTH_DEFAULT;
	mScreenHeight = SCREEN_HEIGHT_DEFAULT;
	mProjectionMatrix = new glm::mat4();
	mProjectionMatrixTexProj = new glm::mat4();
	mProjectionMatrixOrtho = new glm::mat4();
	mBadFactor = 0;
	mBadSwapping = 0;
	mSpeedFactor = 1.0f;
	mTimeDiffWonOrLostWait = 0.0f;
	mNightMode = false;
	mPaused = false;
	mMipMapping = true;
	mTrilinear = true;
	mWireFrame = false;
	mFrustumCulling = true;
	mTransparency = true;
	mLightningVaoOnlyUpdate = false;
	mMousePosition = new glm::ivec2();
	mCursorLastPos = new glm::ivec2();
	mCursorFactor = 1.0;
	mState = Scene::States::Init;
	init();
}

Scene::~Scene(void)
{
	//releaseVboVao(vbo_id_ground, &vao_id_ground);
	//releaseVboVao(vbo_id_triangles, &vao_id_triangles);
	//releaseVboVao(vbo_id_skybox, &vao_id_skybox);
	//releaseVboVao(vbo_id_lightning, &vao_id_lightning);
}

bool Scene::init(void)
{
	srand ( (unsigned int)time(NULL) );	
	
	*this->mProjectionMatrixTexProj = glm::translate(0.5f, 0.5f, 0.0f) * glm::scale(0.5f, 0.5f, 1.0f) * glm::perspective(30.0f,1.0f,1.0f,10.0f);
	this->mSoundInterface = new SoundInterface();
	this->mCamera = new Camera();
	this->mLight = new GameObject(GameObject::Types::Other);
	this->mLightning = new GameObject(GameObject::Types::Other);
	this->mSkybox = new GameObject(GameObject::Types::Other);
	*mSkybox->mScale = glm::vec3(3.5f,3.5f,3.5f);
	mSectors = new Sector*[SECTOR_COUNT];

	this->mGroundParts = new GameObject*[FIELDSPERROW * FIELDSPERROW];
	for (int i = 0, wid = -FIELDSPERROW/2;wid<FIELDSPERROW/2; wid++, i++)
	{
		for (int j = 0, len = -FIELDSPERROW/2;len<FIELDSPERROW/2; len++, j++)
		{
			mGroundParts[(i*FIELDSPERROW) + j] = new GameObject(GameObject::Types::Ground);
			*mGroundParts[(i*FIELDSPERROW) + j]->mPosition = glm::vec3(wid * GAMEFIELD_WIDTH / FIELDSPERROW * 1.0f,0.0f,len * GAMEFIELD_LENGTH / FIELDSPERROW * 1.0f);
			*mGroundParts[(i*FIELDSPERROW) + j]->mScale = glm::vec3(GAMEFIELD_WIDTH / 2 / FIELDSPERROW, GAMEFIELD_WIDTH / 2 / FIELDSPERROW, GAMEFIELD_LENGTH / 2 / FIELDSPERROW);
		}
	}
	this->mGroundBorderParts = new GameObject*[4];
	glm::vec3 scaleGroundParts = glm::vec3(552.61f, 245.0f, 552.61f);
	float translateGroundParts = GAMEFIELD_LENGTH/2 - scaleGroundParts.x *2.41f;
	for (int i=0; i<4 ;i++)
	{
		mGroundBorderParts[i] = new GameObject(GameObject::Types::Other);
		*mGroundBorderParts[i]->mScale = scaleGroundParts;
		int x=0,z=0;
		if (i == 0) { z = -1; }
		if (i == 1) { x = -1; }
		if (i == 2) { z = +1; }
		if (i == 3) { x = +1; }
		*mGroundBorderParts[i]->mPosition = glm::vec3(translateGroundParts * x, +0.0f, translateGroundParts * z);
		mGroundBorderParts[i]->mRotation->y = 90.0f*i;
	}
	this->mInfoPanel = new GameObject(GameObject::Types::Other);

	//overlay items
	this->mMousePointer = new GameObject(GameObject::Types::Overlay);

	mLightning->mVisible = false;
	*mCamera->mPosition = glm::vec3(0.0f,CAMERA_HEIGHT_DEFAULT,CAMERA_HEIGHT_DEFAULT);
	*mCamera->mDirection = glm::normalize(glm::vec3(0.0f,-0.5f,-0.5f));

	*mLight->mPosition = glm::vec3(0.0f,0.0f,0.0f);
	mLightning->mIgnoreTransformations = true;

	mShaderActive = NULL;
	mShaderBlinnphong = new Shader(pathtoshaders + "blinnphongshader");
	mShaderSkybox = new Shader(pathtoshaders + "skyboxshader");
	mShaderFbo = new Shader(pathtoshaders + "fboshader");
	mShaderOverlay = new Shader(pathtoshaders + "overlayshader");
		
	if ((!mShaderBlinnphong) || (!mShaderSkybox) || (!mShaderOverlay)) {
		cerr << "Could not compile shader program." << endl;
		system("pause");
		return false;
	}
	mShaderBlinnphong->bind_frag_data_location("vFragColor");
	mShaderSkybox->bind_frag_data_location("vFragColor");
	mShaderOverlay->bind_frag_data_location("vFragColor");
	mShaderFbo->bind_frag_data_location("vFragColor");

	generateRandom();

	mModelTree = new Model();
	mModelGround = new Model();
	mModelGroundBorder = new Model();
	mModelInfoPanel = new Model();
	mModelMoai = new Model();
	mModelPeopleGood = new Model();
	mModelPeopleBad = new Model();
	mModelLight = new Model();
	mModelSkybox = new Model();
	mModelLightning = new Model();
	mModelHolyGrenade = new Model();
	mModelHolyGrenadeBorder = new Model();	
	mModelStoneWall = new Model();
	
	mModelOverlay = new Model();

	initVboVao(*mShaderSkybox, mModelSkybox->mVboId, &mModelSkybox->mVaoId, skybox_positions, NULL, NULL, sizeof(skybox_positions), GL_STATIC_DRAW, false);
	initVboVao(*mShaderBlinnphong, mModelGround->mVboId, &mModelGround->mVaoId, ground_positions, calculateNormals(ground_positions, sizeof(ground_positions) / sizeof(float)), ground_texture_coordinates, sizeof(ground_positions), GL_STATIC_DRAW, false );	
	initVboVao(*mShaderBlinnphong, mModelInfoPanel->mVboId, &mModelInfoPanel->mVaoId, ground_positions, NULL, ground_texture_coordinates, sizeof(ground_positions), GL_STATIC_DRAW, false );
	initVboVao(*mShaderBlinnphong, mModelLight->mVboId, &mModelLight->mVaoId, triangle_positions, NULL, triangle_texture_coordinates, sizeof(triangle_positions), GL_STATIC_DRAW, false);

	//init overlay objects
	initVboVao(*mShaderOverlay, mModelOverlay->mVboId, &mModelOverlay->mVaoId, overlay_positions, NULL, overlay_texture_coordinates, sizeof(overlay_positions), GL_STATIC_DRAW, false );


	initVboVaoFromModel(*mShaderBlinnphong, mModelHolyGrenade, pathtomodels + "holygrenade2.obj", GL_STATIC_DRAW);	
	initVboVaoFromModel(*mShaderBlinnphong, mModelHolyGrenadeBorder, pathtomodels + "holygrenadeborder2.obj", GL_STATIC_DRAW);	
	initVboVaoFromModel(*mShaderBlinnphong, mModelStoneWall, pathtomodels + "wall7.obj", GL_STATIC_DRAW);
	initVboVaoFromModel(*mShaderBlinnphong, mModelPeopleBad, pathtomodels + "badbobob3.obj", GL_STATIC_DRAW);	
	initVboVaoFromModel(*mShaderBlinnphong, mModelPeopleGood, pathtomodels + "goodbobob2.obj", GL_STATIC_DRAW);	
	initVboVaoFromModel(*mShaderBlinnphong, mModelTree, pathtomodels + "MediumPolyTree2.obj", GL_STATIC_DRAW);	
	initVboVaoFromModel(*mShaderBlinnphong, mModelMoai, pathtomodels + "moai_duplicated.obj", GL_STATIC_DRAW);	
	initVboVaoFromModel(*mShaderBlinnphong, mModelGroundBorder, pathtomodels + "groundborder.obj", GL_STATIC_DRAW);	

	//initVboVaoFromModel(*mShaderBlinnphong, mModelHolyGrenade, pathtomodels + "sphere.obj", GL_STATIC_DRAW);	
	//initVboVaoFromModel(*mShaderBlinnphong, mModelHolyGrenadeBorder, pathtomodels + "sphere.obj", GL_STATIC_DRAW);	
	//initVboVaoFromModel(*mShaderBlinnphong, mModelStoneWall, pathtomodels + "sphere.obj", GL_STATIC_DRAW);
	//initVboVaoFromModel(*mShaderBlinnphong, mModelPeopleBad, pathtomodels + "sphere.obj", GL_STATIC_DRAW);	
	//initVboVaoFromModel(*mShaderBlinnphong, mModelPeopleGood, pathtomodels + "sphere.obj", GL_STATIC_DRAW);	
	//initVboVaoFromModel(*mShaderBlinnphong, mModelTree, pathtomodels + "sphere.obj", GL_STATIC_DRAW);	
	//initVboVaoFromModel(*mShaderBlinnphong, mModelMoai, pathtomodels + "sphere.obj", GL_STATIC_DRAW);	
	//initVboVaoFromModel(*mShaderBlinnphong, mModelGroundBorder, pathtomodels + "sphere.obj", GL_STATIC_DRAW);	

	mModelGround->createTextureHandles(1);
	mModelGround->mTextureHandles[0] = load2dTexture(pathtotextures + "ground/ground2halfglow.tga");

	mModelGroundBorder->createTextureHandles(1);
	mModelGroundBorder->mTextureHandles[0] = load2dTexture(pathtotextures + "ground/groundborderglow.tga");

	mModelInfoPanel->createTextureHandles(3);
	mModelInfoPanel->mTextureHandles[0] = load2dTexture(pathtotextures + "infopanel/infocontrols.tga");
	mModelInfoPanel->mTextureHandles[1] = load2dTexture(pathtotextures + "infopanel/infowin.tga");
	mModelInfoPanel->mTextureHandles[2] = load2dTexture(pathtotextures + "infopanel/infolose.tga");

	mModelOverlay->createTextureHandles(2);
	mModelOverlay->mTextureHandles[0] = load2dTexture(pathtotextures + "overlay/mousepointer.tga");
	mModelOverlay->mTextureHandles[1] = load2dTexture(pathtotextures + "overlay/bubble.tga");
	
	mModelPeopleGood->createTextureHandles(1);
	mModelPeopleGood->mTextureHandles[0] = load2dTexture(pathtotextures + "peoplegood/goodbobob.tga");

	mModelPeopleBad->createTextureHandles(1);
	mModelPeopleBad->mTextureHandles[0] = load2dTexture(pathtotextures + "peoplebad/badbobob2.tga");
	
	mModelTree->createTextureHandles(3);
	mModelTree->mTextureHandles[0] = load2dTexture(pathtotextures + "tree/leaf3.tga");
	mModelTree->mTextureHandles[1] = load2dTexture(pathtotextures + "tree/leafalpha.tga");
	mModelTree->mTextureHandles[2] = load2dTexture(pathtotextures + "tree/bark.tga");

	mModelMoai->createTextureHandles(1);
	mModelMoai->mTextureHandles[0] = load2dTexture(pathtotextures + "moai/moai.tga");
	
	mModelLightning->createTextureHandles(6);
	mModelLightning->mTextureHandles[0] = load2dTexture(pathtotextures + "lightning/lightning5.tga");
	mModelLightning->mTextureHandles[1] = load2dTexture(pathtotextures + "lightning/lightningalpha5.tga");	
	mModelLightning->mTextureHandles[2] = load2dTexture(pathtotextures + "lightning/lightning2.tga");
	mModelLightning->mTextureHandles[3] = load2dTexture(pathtotextures + "lightning/lightningalpha2.tga");	
	mModelLightning->mTextureHandles[4] = load2dTexture(pathtotextures + "projection/projection2.tga", true);
	mModelLightning->mTextureHandles[5] = load2dTexture(pathtotextures + "projection/projection1.tga", true);

	mModelStoneWall->createTextureHandles(2);
	mModelStoneWall->mTextureHandles[0] = load2dTexture(pathtotextures + "stonewall/stonegood3.tga");
	mModelStoneWall->mTextureHandles[1] = load2dTexture(pathtotextures + "stonewall/stonebad3.tga");

	mModelHolyGrenade->createTextureHandles(2);
	mModelHolyGrenade->mTextureHandles[0] = load2dTexture(pathtotextures + "holygrenade/holygrenade2.tga");
	mModelHolyGrenade->mTextureHandles[1] = load2dTexture(pathtotextures + "holygrenade/holygrenade3.tga");

	mModelHolyGrenadeBorder->mUseTexture = 0;
	*mModelHolyGrenadeBorder->mDiffuseColor = glm::vec4(0.7f,0.5f,0.3f, 1.0f);

	mModelInfoPanel->mUseLight = 0;

	string pathtoskybox = pathtotextures + "skyboxgood/";
	string pathToCubeMap[] = {pathtoskybox + "xpos.tga", pathtoskybox + "xneg.tga", pathtoskybox + "ypos.tga", pathtoskybox + "yneg.tga", pathtoskybox + "zpos.tga", pathtoskybox + "zneg.tga"};
	mModelSkybox->mTextureHandles = new int[1];
	mModelSkybox->mTextureHandles[0] = loadCubeMap(pathToCubeMap);

	mModelPeopleGood->mMaterialFactor = 0.2f;
	mModelPeopleBad->mMaterialFactor = 0.2f;
	
	mModelTree->mMaterialFactor = 0.3f;
	mModelTree->mUseTexture = 2;
	
	mModelMoai->mMaterialFactor = 0.7f;

	mModelLightning->mUseLight = 0;
	mModelLightning->mUseTexture = 3;

	mModelLight->mUseLight = 0;
	mModelLight->mUseTexture = 0;
	*mModelLight->mDiffuseColor = glm::vec4(1.0f, 1.0f, 0.0f, 1.0f);

	mModelSkybox->mUseLight = 0;
	mModelSkybox->mUseTexture = 0;
	
	mModelHolyGrenade->mMaterialFactor = 0.8f; 
	
	mModelStoneWall->mMaterialFactor = 0.0f;
	
	this->mSoundInterface->play(SoundInterface::Sounds::MusicGood);
	this->mSoundInterface->play(SoundInterface::Sounds::SoundHolyGrenadeCrafting);
	
	return true;
	
}

void Scene::generateRandom()
{	
	float factor = 3.0f;
	if (SECTOR_COUNT <= 3) { factor = 6; }
	else if (SECTOR_COUNT <= 7) { factor = 5; }
	else if (SECTOR_COUNT <= 13) { factor = 3; }
	else if (SECTOR_COUNT <= 23) { factor = 3; }
	float randBorder = GAMEFIELD_LENGTH_MINUS_MARGIN_SECTORS / factor;
	//for (int i=1;i<=25;i++)
	//{
	//	int SECTOR_COUNT = i;
	//	float factor =  glm::max(1.0f, 1.0f / SECTOR_COUNT * 40);
	//	float randBorder = GAMEFIELD_LENGTH_MINUS_MARGIN_SECTORS / factor;
	//	cout << SECTOR_COUNT << " " << factor << " " << randBorder << endl;
	//}
	for (int i=0;i<SECTOR_COUNT;i++)
	{
		glm::vec3 *position =  new glm::vec3(0.0f, 0.0f, 0.0f);
		if (i != 0) //first one is always 0
		{  
			bool farenough, nearenough;
			do {
				farenough = true;
				nearenough = false;
				*position = glm::vec3(randFloat(-randBorder,+randBorder), 0.0f, randFloat(-randBorder,+randBorder));
				for (int j=0;j<i;j++) 
				{ 
					float len = glm::length(*position - *mSectors[j]->mPosition);
					if (len < SECTOR_LENGTH * 1.2f)  { farenough = false; } //they are too near
					if (len < SECTOR_LENGTH * 2.0f)  { nearenough = true; }
				}
			} 
			while (farenough == false || nearenough == false);
		}
		//cout << i << endl ;
		printVector(*position);
		mSectors[i] = new Sector(position, i, this);
		mSectors[i]->init();
	}	
	for (int i=0;i<SECTOR_COUNT;i++) { mSectors[i]->generateRandom(); } 
	
}

void Scene::handleShoot(int button)
{	
	if (mPaused)
	{
		togglePause(); //break the pause
		return;
	}
	if ((!mPaused) && (mState != Scene::States::Init))
	{
		if ((button == GLFW_MOUSE_BUTTON_LEFT || button == GLFW_MOUSE_BUTTON_RIGHT))
		{
			//calc lightning position
			glm::vec3 unproj = glm::unProject(glm::vec3(mMousePosition->x, mScreenHeight - mMousePosition->y, 0),*mCamera->mViewMatrix, *mProjectionMatrix, glm::vec4(0,0,mScreenWidth,mScreenHeight)) - *(mCamera->mPosition);		
			float t = -mCamera->mPosition->y / unproj.y;
			glm::vec3 shootPosition = (*mCamera->mPosition) + t * unproj;
			*mLightning->mPosition = shootPosition;
			mLightning->mVisible = true;
			mLightningAlive = 0.0f;
			if (button == GLFW_MOUSE_BUTTON_LEFT) { mLightning->mType = GameObject::Types::PersonBad; mSoundInterface->play(SoundInterface::Sounds::SoundLightningShoot);}
			else if (button == GLFW_MOUSE_BUTTON_RIGHT) { mLightning->mType = GameObject::Types::PersonGood; mSoundInterface->play(SoundInterface::Sounds::SoundLightningPick);}
		
			//get selected object and update object state		
			if (mSelectedObject != nullptr)
			{
				GameObject * item = mSelectedObject;
				mSelectedObject = nullptr;
				if (button == GLFW_MOUSE_BUTTON_LEFT) 
				{
					if ((item->mType == GameObject::Types::PersonGood || item->mType == GameObject::Types::PersonBad) && item->mState != GameObject::States::Dying) 
					{ 
						if (item->mState == GameObject::States::Crafting) { item->mSector->mCountCraftingPeople--; }				
						item->mState = GameObject::States::Dying;
						mSoundInterface->play(SoundInterface::Sounds::SoundPeopleShot);
					}
					else if (item->mType == GameObject::Types::HolyGrenade && item->mState == GameObject::States::CraftingFinished) 
					{ 
						item->mState = GameObject::States::MovingToTree;
						this->mSoundInterface->play(SoundInterface::Sounds::SoundHolyGrenadeRolling);
					}
				}
				else if (button == GLFW_MOUSE_BUTTON_RIGHT)
				{
					if ((item->mType == GameObject::Types::PersonGood || item->mType == GameObject::Types::PersonBad) && item->mState == GameObject::States::Moving) 
					{ 
						if (item->mType == GameObject::Types::PersonGood) { item->mState = GameObject::States::MovingToCraft; }
						item->setDirectionXZ(item->mSector->mHolyGrenade->getPositionXZ() - *item->mPosition); 
					}
					else if (item->mType == GameObject::Types::Tree && item->mState != GameObject::States::Dead)
					{
						mCamera->mParent = item;
					}
				}
				//for (int row=0;row< SECTOR_ROWS;row++) { for (int col=0;col<SECTOR_COLS;col++) { if (mSectors[SECTOR_COLS * row + col]->withinBorders(&shootPosition)) { mSectors[SECTOR_COLS * row + col]->handleShoot(button, mousePosition); } } }
			}
		}
		//else if (button == GLFW_MOUSE_BUTTON_MIDDLE)
		//{
		//	  mCamera->mHeightFixed = !mCamera->mHeightFixed;
		//}
	}
	//double timeAfter = glfwGetTime();				
	//cout << "\r\nElapsed Time: (s) " << timeAfter - timeBefore;
}

void Scene::resize(int x, int y)
{
	if (x != 0 && y != 0)
	{
		cout << "fbo recreate" << endl;
		mScreenWidth = x;
		mScreenHeight = y;
		glViewport(0, 0, mScreenWidth, mScreenHeight);
		*mProjectionMatrix = glm::perspective(60.0f,(float)mScreenWidth/(float)mScreenHeight,1.0f,glm::max(GAMEFIELD_WIDTH,GAMEFIELD_LENGTH) * 3.0f);
		//*mProjectionMatrixOrtho = glm::ortho(0.0f, (float)mScreenWidth, (float)mScreenHeight, 0.0f, 0.0f, -1.0f);
		//cout << mScreenWidth << " " << mScreenHeight << endl;
		initFbo(mFboId,mFboImgId, mScreenWidth, mScreenHeight);
	}
}

bool Scene::checkFrustumCulling(GameObject *gameObject)
{
	if (mFrustumCulling)
	{
		//cout << endl << endl;
		//cout << "left: " << mViewPlanes[0].distance(*mLight->mPosition) << endl;
		//cout << "right: " << mViewPlanes[1].distance(*mLight->mPosition) <<  endl;
		//cout << "top: " << mViewPlanes[2].distance(*mLight->mPosition) <<  endl;
		//cout << "bottom: " << mViewPlanes[3].distance(*mLight->mPosition) <<  endl;
		//cout << "near: " << mViewPlanes[4].distance(*mLight->mPosition) << endl;
		//cout << "far: " << mViewPlanes[5].distance(*mLight->mPosition) << endl;
		float distance = 0;
		for (int i=0;i<6;i++) { distance = glm::min(mPlanes[i].getDistance(*gameObject->mPosition), distance); }
		//cout << "min: " << distance << endl;
		if (distance + gameObject->mRadiusBoundingSphere < 0) 
		{ 
			mCountCulledObjects++;
			return false; 
		}
		mCountDrawnObjects++;
	}
	return true;
}

void Scene::toggleMipMapping(void)
{
	this->mMipMapping = !this->mMipMapping;
	Model::changeTextures(this->mMipMapping, this->mTrilinear);
	cout << "MipMapping: " << this->mMipMapping << endl;
}

void Scene::toggleTrilinear(void)
{
	this->mTrilinear = !this->mTrilinear;
	Model::changeTextures(this->mMipMapping, this->mTrilinear);
	cout << "TriLinear: " << this->mTrilinear << endl;
}

void Scene::toggleWireFrame(void)
{
	this->mWireFrame = !this->mWireFrame;
	cout << "WireFrame: " << this->mWireFrame << endl;
}

void Scene::toggleTransparency(void)
{
	this->mTransparency = !this->mTransparency;
	cout << "Transparency: " << this->mTransparency << endl;	
}

void Scene::toggleFrustumCulling(void)
{
	this->mFrustumCulling = !this->mFrustumCulling;
	cout << "FrustumCulling: " << this->mFrustumCulling << endl;	
}

void Scene::toggleNightMode(void)
{
	mNightMode = !mNightMode;
	if (mNightMode) { *mLight->mPosition = glm::vec3(0.0f, 5.0f, 0.0f); }
	else  { *mLight->mPosition = glm::vec3(0.0f,GAMEFIELD_LENGTH / 4.0f,0.0f); }
}

void Scene::togglePause(glm::vec3 *cameraPosition)
{
	mPaused = !mPaused;
	static glm::vec3 camposbeforepause, camdirbeforepause;
	if (mPaused)
	{
		camposbeforepause = *mCamera->mPosition;
		camdirbeforepause = *mCamera->mDirection;
		
		if (cameraPosition != NULL) { *mCamera->mPosition = *cameraPosition; }
		else { *mCamera->mPosition = glm::vec3(0.0f, 200.0f, 0.0f); }
		*mCamera->mDirection = glm::normalize(glm::vec3(0,-1,-1));

		(*mInfoPanel->mPosition) = (*mCamera->mPosition) + (*mCamera->mDirection) * 2.0f;
		this->mInfoPanel->mRotation->z = 45.0f;
		//this->mInfoPanel->mRotation->y = mCamera->mRotation->y;
		mInfoPanel->mVisible = true;
		mMousePointer->mVisible = false;
		glfwEnable(GLFW_MOUSE_CURSOR);
	}
	else
	{
		*mCamera->mPosition = camposbeforepause;
		*mCamera->mDirection = camdirbeforepause;
		mInfoPanel->mVisible = false;
		mMousePointer->mVisible = true;
		glfwDisable(GLFW_MOUSE_CURSOR);
	}
}

void Scene::playSingleSoundDistance(float maxLength, glm::vec3 positionObject, SoundInterface::Sounds soundType, bool debugOutput)
{
	if (!mSoundInterface->isPlaying(soundType))
	{
		float len = glm::length(*mCamera->mPosition - positionObject);
		if (len <= maxLength)
		{
			float volume =  glm::clamp(1 - (len / maxLength), 0.0f, 1.0f );
			mSoundInterface->changeVolume(soundType, volume);
			mSoundInterface->play(soundType);
			if (debugOutput) {cout << "playing, length = " << len << ", volume = " << volume << endl; }
		}
		else { if (debugOutput) {cout << "would play, but distance is too far: " << len << ", maxlength = " << maxLength << endl;}}
	}
	else { if (debugOutput) {cout << "would play, but is currently playing" << endl; }}
}


void Scene::update(float timeDiff)
{
	static float firstUpdate = true;

	//update speedfactor
	if  (glfwGetKey(GLFW_KEY_KP_ADD)  && glfwGetKey(GLFW_KEY_RCTRL)) { mSpeedFactor += 0.1f;  }
	if  (glfwGetKey(GLFW_KEY_KP_SUBTRACT)  && glfwGetKey(GLFW_KEY_RCTRL)) { mSpeedFactor -= 0.1f; }
	if  (glfwGetKey(GLFW_KEY_KP_0)  && glfwGetKey(GLFW_KEY_RCTRL)) { mSpeedFactor = 1.0f; }
	if (firstUpdate) { timeDiff = 0.0001; }
	updateCamera(timeDiff / mSpeedFactor, firstUpdate);
	if (!mPaused)
	{
		//updateMousePosition(timeDiff);
		updateObjectPicking(timeDiff);
		updateSkyBox(timeDiff);
		updateBorderParts(timeDiff);
		updateLight(timeDiff);
		updateLightning(timeDiff);
		
		updateSectors(timeDiff, firstUpdate);
		updateBadFactor(timeDiff);
		updateSound(timeDiff);
	}
	firstUpdate = false;
	if (mState == Scene::States::Won || mState == Scene::States::Lost)
	{
		mTimeDiffWonOrLostWait += timeDiff;
		if (mTimeDiffWonOrLostWait > 6.0f && mTimeDiffWonOrLostWait < 27.0f) { mTimeDiffWonOrLostWait = 28.0f; togglePause();  }
	}
	//if (firstUpdate) 
	//{
	//	//this->togglePause();
	//	firstUpdate = false;
	//}
	//updateInfoPanel(timeDiff); //info panel is always upd
}

void Scene::updateSkyBox(float timeDiff)
{
	*mSkybox->mPosition = *mCamera->mPosition;
	//*mSkybox->mPosition = *mCamera->mPosition + glm::normalize(glm::vec3(mCamera->mDirection->x, 0.0f, mCamera->mDirection->z)) * (GAMEFIELD_LENGTH/2.5f) ;
}

/*
void Scene::updateMousePosition(float timeDiff)
{
	int mousex, mousey;
	glfwGetMousePos(&mousex, &mousey);
	mMousePosition->x = mousex;
	mMousePosition->y = mousey;
}
*/
void Scene::updateBorderParts(float timeDiff)
{
	//static float tweakscalex = 552.61f, tweakscaley = 245.0f, tweakposy = 13.7f, tweaktrans = 2.41f; //tweakposy = 31.1 for no z-fight
	//float translateGroundParts = GAMEFIELD_LENGTH/2 - tweakscalex * tweaktrans;

	//if (glfwGetKey(GLFW_KEY_F3)) {tweakscalex+=0.01f; cout << tweakscalex << endl;}
	//if (glfwGetKey(GLFW_KEY_F4)) {tweakscalex-=0.01f; cout << tweakscalex << endl;}
	//if (glfwGetKey(GLFW_KEY_F5)) {tweakscaley+=5; cout << tweakscaley << endl;}
	//if (glfwGetKey(GLFW_KEY_F6)) {tweakscaley-=5; cout << tweakscaley << endl;}
	//if (glfwGetKey(GLFW_KEY_F7)) {tweakposy+=0.1f; cout << tweakposy << endl;}
	//if (glfwGetKey(GLFW_KEY_F8)) {tweakposy-=0.1f; cout << tweakposy << endl;}
	//if (glfwGetKey(GLFW_KEY_F7)) {tweaktrans+=0.01f; cout << tweaktrans << endl;}
	//if (glfwGetKey(GLFW_KEY_F8)) {tweaktrans-=0.01f; cout << tweaktrans << endl;}
	for (int i=0;i<4;i++)  
	{ 
		mGroundBorderParts[i]->mPosition->y = mCamera->mPosition->y / (GAMEFIELD_MAX / 2) * 100; //clamp pos of the mountains between 0 and 100
		//int x=0,z=0;
		//if (i == 0) { z = -1; }
		//if (i == 1) { x = -1; }
		//if (i == 2) { z = +1; }
		//if (i == 3) { x = +1; }		
		//*mGroundBorderParts[i]->mScale = glm::vec3(tweakscalex,tweakscaley,tweakscalex);  
		//mGroundBorderParts[i]->mPosition->x = translateGroundParts * x;
		//mGroundBorderParts[i]->mPosition->z = translateGroundParts * z;
	}	
}

void Scene::updateCamera(float timeDiff, bool firstUpdate)
{
	if (!mPaused)
	{
		camchanged = false;
		float cammove = timeDiff * CAMERA_MOVE_PERSECOND;
		float camrotatey = timeDiff * CAMERA_ROTATEY_PERSECOND;
		float camrotatex = timeDiff * CAMERA_ROTATEX_PERSECOND;	
		static int mousewheelposold = 0;
		static float mousewheelcounter = 0.0f;	
		static float maxTimeForInitialFly = 0.0f;
		if (mState == Scene::States::Init) //initial cam fly
		{
			if (firstUpdate)
			//if ( ((glm::length(*mCamera->mPosition) < 1000 && glm::abs(mCamera->mDirection->x) < 0.01f && mCamera->mDirection->z <= -0.5f)) || maxTimeForInitialFly >= 40.0f || glfwGetKey(GLFW_KEY_F1)) 
			{		
				//mSoundInterface->stop(SoundInterface::Sounds::SoundHolyGrenadeCrafting);
				cout << maxTimeForInitialFly << endl;
				//cout << "light is: " << mLight->mPosition->y << " light should: " << GAMEFIELD_LENGTH / 6;
				//printVector(*mCamera->mPosition); 
				//printVector(*mCamera->mDirection);
				*mCamera->mPosition = glm::vec3(0.0f, 576.0f, 576.0f);
				*mCamera->mDirection = glm::normalize(glm::vec3(0.0f, -1.0f, -1.0f));
				mLight->mPosition->y = GAMEFIELD_LENGTH / 6;
				mState = Scene::States::Normal; 
				togglePause(mCamera->mPosition); 
			}
			else
			{
				maxTimeForInitialFly += timeDiff;
				mLight->mPosition->y += timeDiff * 25;		
				if (maxTimeForInitialFly >= 3.0f)
				{
					*mCamera->mDirection = (glm::mat3(glm::rotate(-timeDiff * 20,0.0f,1.0f,0.0f)) * (*mCamera->mDirection));
					*mCamera->mPosition =  (glm::length(*mCamera->mPosition / (1+ timeDiff/10)) * (*mCamera->mDirection) * -1.0f);
				}
			}
		}
		else
		{
			if (mCamera->mParent == NULL) //camera has no parented object
			{
				//camera y-axis rotation
				if (glfwGetKey(GLFW_KEY_LEFT) || glfwGetKey('A')) {	*mCamera->mDirection = (glm::mat3(glm::rotate(+camrotatey,0.0f,1.0f,0.0f)) * (*mCamera->mDirection)); mCamera->mRotation->y += camrotatey; camchanged = true; }
				if (glfwGetKey(GLFW_KEY_RIGHT) || glfwGetKey('D')){	*mCamera->mDirection = (glm::mat3(glm::rotate(-camrotatey,0.0f,1.0f,0.0f)) * (*mCamera->mDirection)); mCamera->mRotation->y -= camrotatey; camchanged = true; }

				//camera movement
				if (mCamera->mHeightFixed)
				{
					if (glfwGetKey(GLFW_KEY_UP) || glfwGetKey('W')){ *(mCamera->mPosition) += glm::normalize(glm::vec3(mCamera->mDirection->x, 0.0f, mCamera->mDirection->z)) * cammove; camchanged = true; }
					if (glfwGetKey(GLFW_KEY_DOWN) || glfwGetKey('S')) { *(mCamera->mPosition) -= glm::normalize(glm::vec3(mCamera->mDirection->x, 0.0f, mCamera->mDirection->z)) * cammove; camchanged = true; }
				}
				else
				{
					if (glfwGetKey(GLFW_KEY_UP) || glfwGetKey('W')){ *(mCamera->mPosition) += *mCamera->mDirection * cammove; camchanged = true; }
					if (glfwGetKey(GLFW_KEY_DOWN) || glfwGetKey('S')) { *(mCamera->mPosition) -= *mCamera->mDirection * cammove; camchanged = true; }
				}
				//camera height
				if  (glfwGetKey(GLFW_KEY_SPACE)) { mCamera->mPosition->y += cammove; camchanged = true;}
				if  (glfwGetKey('C')  || glfwGetKey(GLFW_KEY_LCTRL)) { mCamera->mPosition->y -= cammove; camchanged = true;}
				if (mCamera->mPosition->y < CAMERA_HEIGHT_MINIMUM) { mCamera->mPosition->y = CAMERA_HEIGHT_MINIMUM; }

				int mousewheelposnew = glfwGetMouseWheel();
				if (mousewheelposold != mousewheelposnew)
				{
					mousewheelcounter += (mousewheelposnew - mousewheelposold) / 8.0f;
					mousewheelposold = mousewheelposnew;
				}
				//camera x and z-axis rotation
				if (mousewheelcounter > 0) {mousewheelcounter-=timeDiff; *(mCamera->mDirection) = (glm::mat3(glm::rotate(+camrotatex ,-(mCamera->mDirection->z),0.0f,mCamera->mDirection->x))) * (*mCamera->mDirection); mCamera->mRotation->z += camrotatex; camchanged = true;}	
				if (mousewheelcounter < 0) {mousewheelcounter+=timeDiff; *(mCamera->mDirection) = (glm::mat3(glm::rotate(-camrotatex ,-(mCamera->mDirection->z),0.0f,mCamera->mDirection->x))) * (*mCamera->mDirection); mCamera->mRotation->z -= camrotatex;camchanged = true;}
	
				//camera->if too much undo the rotation
				if (mCamera->mDirection->y >= 0.95) { *(mCamera->mDirection) = (glm::mat3(glm::rotate(-camrotatex,-(mCamera->mDirection->z),0.0f,mCamera->mDirection->x))) * (*mCamera->mDirection); mCamera->mRotation->z -= camrotatex; camchanged = false;}	
				if (mCamera->mDirection->y <= -0.95)  { *(mCamera->mDirection) = (glm::mat3(glm::rotate(+camrotatex,-(mCamera->mDirection->z),0.0f,mCamera->mDirection->x))) * (*mCamera->mDirection); mCamera->mRotation->z += camrotatex; camchanged = false;}
			}
			else //camera has parented object
			{
				glm::vec3 posParent = *mCamera->mParent->mPosition + glm::vec3(0.0f,80.0f,0.0f);
				*mLightning->mPosition = *mCamera->mParent->mPosition;
				if (glm::length(posParent - *mCamera->mPosition) < 250) {mCamera->mParent = NULL; }
				else
				{
					glm::vec3 directionShould = glm::normalize(posParent - *mCamera->mPosition);
					glm::vec3 directionIs = *mCamera->mDirection;
					glm::vec3 directionDiff = (directionShould - directionIs);
					glm::vec3 directionNew = glm::normalize(glm::vec3(directionDiff.x * timeDiff, directionDiff.y * timeDiff *2.5f, directionDiff.z * timeDiff) * 2.0f + directionIs);
					*mCamera->mDirection = directionNew;
					*(mCamera->mPosition) += *mCamera->mDirection * CAMERA_MOVE_FROMPARENT_PERSECOND * timeDiff;
				}
			}
		}
	}
	//update viewmatrix and frustum planes
	*(mCamera->mViewMatrix) = glm::lookAt(*(mCamera->mPosition),(*(mCamera->mPosition)) + (*(mCamera->mDirection)),glm::vec3(0.0,+1.0,0.0));
	//glm::vec3 up = glm::vec3(0.0,+1.0,0.0);
	//glm::vec3 right1 = glm::cross(glm::vec3(0,1,0)
	glm::vec3 up = glm::mat3(glm::rotate(90.0f, -mCamera->mDirection->z, 0.0f,mCamera->mDirection->x)) * (*(mCamera->mDirection));
	glm::vec3 d = *(mCamera->mDirection);
	if (glfwGetKey('F')) { cout << "dir: "; printVector(d); cout << "up:  "; printVector (up); }
	glm::vec3 p = *(mCamera->mPosition);

	// should be stored extra: *mProjectionMatrix = glm::perspective(60.0f,(float)mScreenWidth/(float)mScreenHeight,.1f,glm::max(GAMEFIELD_WIDTH,GAMEFIELD_LENGTH) * glm::sqrt(2.0f));
	float farDist = glm::max(GAMEFIELD_WIDTH,GAMEFIELD_LENGTH) * glm::sqrt(2.0f);
	float nearDist = 1.0f;
	float ratio = (float)mScreenWidth/(float)mScreenHeight;
	float Hnear = 2 * tan(glm::radians(60.0f) / 2.f) * nearDist;
	float Wnear = Hnear * ratio;
	float Hfar = 2 * tan(glm::radians(60.0f) / 2.f) * farDist;
	float Wfar = Hfar * ratio;

	glm::vec3 right = -glm::cross(up,d);

	glm::vec3 fc = p + d * farDist;

	glm::vec3 ftl = fc + (up * Hfar/2.f) - (right * Wfar/2.f);
	glm::vec3 fbl = fc - (up * Hfar/2.f) - (right * Wfar/2.f);
	glm::vec3 fbr = fc - (up * Hfar/2.f) + (right * Wfar/2.f);
	glm::vec3 ftr = fc + (up * Hfar/2.f) + (right * Wfar/2.f);

	glm::vec3 nc = p + d * nearDist;

	glm::vec3 ntl = nc + (up * Hnear/2.f) - (right * Wnear/2.f);
	glm::vec3 nbl = nc - (up * Hnear/2.f) - (right * Wnear/2.f);
	glm::vec3 nbr = nc - (up * Hnear/2.f) + (right * Wnear/2.f);
	glm::vec3 ntr = nc + (up * Hnear/2.f) + (right * Wnear/2.f);

	//left plane
	mPlanes[0].set3Points(ntl,nbl,fbl);
	//right plane
	mPlanes[1].set3Points(nbr,ntr,fbr);
	//top plane
	mPlanes[2].set3Points(ntr,ntl,ftl);
	//bottom plane
	mPlanes[3].set3Points(nbl,nbr,fbr);
	//near plane
	mPlanes[4].set3Points(ntl,ntr,nbr);
	//far plane
	mPlanes[5].set3Points(ftr,ftl,fbl);
}

void Scene::updateLight(float timeDiff)
{	
	//mLight->mPosition->y = 10.0f;
	//if (glfwGetKey('I')) { *(mLight->mPosition) += glm::normalize(glm::vec3(mCamera->mDirection->x, 0.0f, mCamera->mDirection->z)) * timeDiff * 200.0f; }
	//if (glfwGetKey('K')) { *(mLight->mPosition) -= glm::normalize(glm::vec3(mCamera->mDirection->x, 0.0f, mCamera->mDirection->z)) * timeDiff * 200.0f; }
	//if (glfwGetKey('J')) { *(mLight->mPosition) += glm::mat3(glm::rotate(+90.0f, 0.0f, 1.0f, 0.0f)) * glm::normalize(glm::vec3(mCamera->mDirection->x, 0.0f, mCamera->mDirection->z)) * timeDiff * 200.0f; }
	//if (glfwGetKey('L')) { *(mLight->mPosition) += glm::mat3(glm::rotate(-90.0f, 0.0f, 1.0f, 0.0f)) * glm::normalize(glm::vec3(mCamera->mDirection->x, 0.0f, mCamera->mDirection->z)) *timeDiff * 200.0f; }
	//if (glfwGetKey(GLFW_KEY_F5)) { mLight->mPosition->y += cammove; }
	//if (glfwGetKey(GLFW_KEY_F6)) { mLight->mPosition->y -= cammove; }	
	
	if (!mNightMode && mState != Scene::States::Init)
	{
		glm::vec3 lposnew = *mCamera->mPosition - glm::normalize(glm::vec3(mCamera->mDirection->x, 0.0f, mCamera->mDirection->z)) * 10.0f;
		mLight->mPosition->x = lposnew.x;
		mLight->mPosition->z = lposnew.z;
	}
}

void Scene::updateLightning(float timeDiff)
{
	//lightning
	if (mLightning->mVisible)
	{
		float lightningmove = timeDiff * LIGHTNING_MOVE_PERSECOND;	
		mLightningAlive += timeDiff;
		static float summove = 0;
		summove -=lightningmove;
		glm::vec3 lightningcampos1 = glm::vec3(glm::inverse(*mCamera->mViewMatrix) * glm::vec4(1.0f,2.0f,-2.0f,1.0f));
		glm::vec3 lightningcampos2 = glm::vec3(glm::inverse(*mCamera->mViewMatrix) * glm::vec4(2.3f,1.3f,-2.0f,1.0f));
	
		glm::vec3 diffvec = (lightningcampos2 - lightningcampos1) / 2.0f;
		*mLightning->mDirection = glm::normalize(*mLightning->mPosition - lightningcampos2);
	
		//cout <<  endl << "campos: "; printVector(lightningcampos1);
		//cout << endl << "floorpos: " ; printVector(*mLightning->mPosition);
		//cout << endl << "direction: "; printVector(*mLightning->mDirection);
		//lightning as 2 triangles
		GLfloat lightning_positions[] = {  							
			lightningcampos1.x, lightningcampos1.y, lightningcampos1.z, +1.0f, 		//v0
			lightningcampos2.x, lightningcampos2.y, lightningcampos2.z, +1.0f, 		//v1
			mLightning->mPosition->x - diffvec.x , mLightning->mPosition->y - diffvec.y, mLightning->mPosition->z - diffvec.z, +1.0f, 		//v2
											
			lightningcampos2.x, lightningcampos2.y, lightningcampos2.z, +1.0f, 		//v1											
			mLightning->mPosition->x + diffvec.x,  mLightning->mPosition->y + diffvec.y, mLightning->mPosition->z + diffvec.z, +1.0f, 		//v3
			mLightning->mPosition->x - diffvec.x , mLightning->mPosition->y - diffvec.y, mLightning->mPosition->z - diffvec.z, +1.0f, 		//v2
		};
		float distance = glm::length(*mLightning->mPosition - *mCamera->mPosition);
		//texture coordinates depend on length of ray
	
		GLfloat lightning_texture_coordinates[] = {	
			0,  0 + summove, //v0
			1,  0 + summove, //v1
			0,  distance/20.0f + summove, //v2

			1,  0 + summove, //v1
			1,  distance/20.0f + summove, //v3
			0,  distance/20.0f + summove, //v2
				};
	
		initVboVao(*mShaderBlinnphong, mModelLightning->mVboId, &mModelLightning->mVaoId, lightning_positions, NULL, lightning_texture_coordinates, sizeof(lightning_positions), GL_DYNAMIC_DRAW, mLightningVaoOnlyUpdate);
		mLightningVaoOnlyUpdate = true;
		if (mLightningAlive >= LIGHTNING_VISIBLE_SECONDS && mCamera->mParent == NULL)
		{
			mSoundInterface->stop(SoundInterface::Sounds::SoundLightningPick);
			mLightningAlive = 0.0f;
			mLightning->mVisible = false;
		}
	}
}

void Scene::updateCursorPosition() 
{
	int xpos;
	int ypos;
	glfwGetMousePos(&xpos, &ypos);
	int deltapos_x = floor((xpos - mCursorLastPos->x) * mCursorFactor);
	int deltapos_y = floor((ypos - mCursorLastPos->y) * mCursorFactor);
	mMousePosition->x = mCursorLastPos->x + deltapos_x;
	mMousePosition->y = mCursorLastPos->y + deltapos_y;
	glfwSetMousePos(mMousePosition->x, mMousePosition->y);
	mCursorLastPos = mMousePosition;
	
	if (PickingHelper::getInstance()->m_showBubbleCursor)
	{
		mMousePointer->setScaleUniform(1.0f); //bubble cursor needs fullscreen-quad
		mMousePointer->setPosition(0.0f,0.0f,0.0f);
	}
	else
	{
		glm::vec2 posNDC = glm::vec2((float)mMousePosition->x / (float)mScreenWidth, (float)(mScreenHeight - mMousePosition->y) / (float)mScreenHeight) * 2.0f - 1.0f;
		float scale = glm::clamp((mCursorFactor -0.5) /2.0f, 0.02, 0.9);
		mMousePointer->setScale(scale,scale *  (float)mScreenWidth/(float)mScreenHeight , 0);
		mMousePointer->setPosition(posNDC.x,posNDC.y, 0.0);
	}
}

void Scene::updateObjectPicking(float timeDiff)
{
	updateCursorPosition();
	//check for unallowed mouse positions
	if (mMousePosition->x < 1 || mMousePosition->x >= mScreenWidth - 1 || mMousePosition->y < 1 || mMousePosition->y >= mScreenHeight -1 ) 
	{ 
		glfwEnable(GLFW_MOUSE_CURSOR); //make the mouse cursor visible if it is out of window
		glfwSetMousePos(mMousePosition->x,mMousePosition->y);
		return; 
	}
	else { glfwDisable(GLFW_MOUSE_CURSOR);  }

	//highlighting for object picking
	for (int i=0;i<(int)sGameObjects.size();i++) { sGameObjects.at(i)->mHighlighted = false; }
		
	int xFrom = mMousePosition->x - PickingHelper::getInstance()->m_pickingbuffer_window_width/2;
	int yFrom = mScreenHeight - mMousePosition->y - PickingHelper::getInstance()->m_pickingbuffer_window_width/2;
	int xTo = xFrom + PickingHelper::getInstance()->m_pickingbuffer_window_width;
	int yTo = yFrom + PickingHelper::getInstance()->m_pickingbuffer_window_width;
	
	int xFromCut = glm::max(0,xFrom);
	int yFromCut = glm::max(0,yFrom);
	int xToCut = glm::min(xTo,mScreenWidth);
	int yToCut = glm::min(yTo,mScreenHeight);
	int width = xToCut - xFromCut;
	int height = yToCut - yFromCut;

	int xRelCenter = PickingHelper::getInstance()->m_pickingbuffer_window_width/2 - (xFromCut - xFrom);
	int yRelCenter = PickingHelper::getInstance()->m_pickingbuffer_window_width/2 - (yFromCut - yFrom);

	mPickBufferFrom = glm::ivec2(xFromCut,yFromCut);
	mPickBufferSize = glm::ivec2(width, height);
	int *objectIds = readPixels(xFromCut,yFromCut,width, height, mFboId, mFboImgId);

	//search window console output
	if (glfwGetKey('O'))
	{
		for (int i=0;i < height; i++)
		{	
			for (int j = 0; j < width; j++)
			{				
				char id [5];
				sprintf(id, "%3d", objectIds[j + width*i]);  
				if (i == yRelCenter && j == xRelCenter) { cout << "  C "; }
				else { cout << id << " "; }

			}
			cout << endl;
		}
		cout << endl << endl;
	}	

	mSelectedObject = nullptr;
	int objectid = 0;
	//checkNN_naive(objectIds,width,height,xRelCenter,yRelCenter, objectid, mDistance);
	//checkNN(objectIds,width,height,xRelCenter,yRelCenter, objectid, mDistance);
	//update mouse scale factor
	
	//cout << endl << endl << "Naive: " << objectid << " " << mDistance << endl;
	//int oldID = objectid;
	//double oldDist = mDistance;
	PickingHelper::getInstance()->checkNN(objectIds,width,height,xRelCenter,yRelCenter, objectid, mDistance);
	mCursorFactor = PickingHelper::getInstance()->scale_by_distance(mDistance);
	//cout << endl << "Totally Optimized: " << objectid << " " << mDistance << endl;
	//cout << "ID match: " << (oldID == objectid) << ", mDistance: " << oldDist << ", " << mDistance << endl;
	//free the pixelbuffermemory
	delete(objectIds);

	//highlight the nearest found item
	if (objectid != 0)
	{
		GameObject *item =  (*(sGameObjects.find(objectid))).second;
			
		if	(	(item->mType == GameObject::Types::PersonGood && item->mState != GameObject::States::Dying) || 
				(item->mType == GameObject::Types::PersonBad && item->mState != GameObject::States::Dying) || 
				(item->mType == GameObject::Types::Tree && item->mState != GameObject::States::Dead) ||
				(item->mType == GameObject::Types::HolyGrenade && item->mState == GameObject::States::CraftingFinished)
			) 
		{ 
			if (PickingHelper::getInstance()->m_showBubbleCursor || mDistance < 0.01 ) { item->mHighlighted = true; mSelectedObject = item; }			
		}
	}
}

void Scene::updateSectors(float timeDiff, bool firstUpdate)
{	
	for (int i=0;i<SECTOR_COUNT;i++) { mSectors[i]->update(timeDiff, firstUpdate); } 
}

void Scene::updateBadFactor(float timeDiff)
{
	
	if (mBadSwapping == 0) //no swapping
	{
		int mBadStateSectors = 0, mWonSectors = 0, mLostSectors = 0;
		for (int i=0;i<SECTOR_COUNT;i++) 
		{ 
			mBadStateSectors += mSectors[i]->mBadState; 
			if (mSectors[i]->mState == Sector::States::Won) { mWonSectors++; }
			if (mSectors[i]->mState == Sector::States::Lost) { mLostSectors++; }
		} 
		if		((int)mBadFactor <= 0 && mBadStateSectors > 0) { mBadSwapping = 1; 	mSoundInterface->play(SoundInterface::Sounds::MusicBad);}
		else if ((int)mBadFactor >= 1 && mBadStateSectors <= 0) { mBadSwapping = -1; mSoundInterface->play(SoundInterface::Sounds::MusicGood);}
		if (mState == Scene::States::Normal)
		{
			if (mWonSectors >= SECTOR_COUNT / 2 + 1) { mState = Scene::States::Won;  }
			else if (mLostSectors >= SECTOR_COUNT / 2 + 1) { mState = Scene::States::Lost;  }
		}
	}
	else 
	{ 
		mBadFactor += mBadSwapping * timeDiff; 
		if (mBadFactor < 0) { mBadFactor = 0; mBadSwapping = 0;}
		if (mBadFactor > 1) { mBadFactor = 1; mBadSwapping = 0;}
	}
}

void Scene::updateSound(float timeDiff)
{
	//updating sound
	float volumeHolyGrenadeCrafting = 0.0f, volumeHolyGrenadeRollingToTree = 0.0f;
	for (int i=0;i<SECTOR_COUNT;i++)  {volumeHolyGrenadeCrafting = glm::max(volumeHolyGrenadeCrafting, mSectors[i]->getVolume(SoundInterface::Sounds::SoundHolyGrenadeCrafting)); } 
	for (int i=0;i<SECTOR_COUNT;i++)  {volumeHolyGrenadeRollingToTree = glm::max(volumeHolyGrenadeRollingToTree, mSectors[i]->getVolume(SoundInterface::Sounds::SoundHolyGrenadeRolling)); } 
	
	mSoundInterface->changeVolume(SoundInterface::Sounds::SoundHolyGrenadeCrafting, volumeHolyGrenadeCrafting);
	mSoundInterface->changeVolume(SoundInterface::Sounds::SoundHolyGrenadeRolling, volumeHolyGrenadeRollingToTree);
	float volumeMax = glm::max(volumeHolyGrenadeCrafting, volumeHolyGrenadeRollingToTree);
	
	mSoundInterface->changeVolume(SoundInterface::Sounds::MusicBad, 0.5f - volumeMax);
	mSoundInterface->changeVolume(SoundInterface::Sounds::MusicGood, 0.5f - volumeMax);
}

void Scene::setShaderActive(Shader *shader)
{
	if (mShaderActive != NULL) { mShaderActive->unbind(); }
	mShaderActive = shader;
	mShaderActive->bind();
}

void Scene::setUniforms(const Model *model, const GameObject *gameobject, const Sector *sector)
{
	glm::mat4 modelMatrix = gameobject->getModelMatrix();
	glm::mat4 modelViewMatrix = (*mCamera->mViewMatrix) * modelMatrix;
	glm::mat4 modelViewProjectionMatrix = (*mProjectionMatrix) * modelViewMatrix;
	//glm::mat3 normalMatrix = glm::transpose(glm::inverse(glm::mat3(modelViewMatrix))); //only necessary if non-uniform scaling applies
	glm::mat3 normalMatrix = glm::mat3(modelViewMatrix);
	
	glUniformMatrix4fv(mShaderActive->get_uniform_location("mvpMatrix"), 1, GL_FALSE, glm::value_ptr(modelViewProjectionMatrix));
	
	if (mShaderActive == mShaderFbo) //to fbo rendering is just the uniform objectid necessary
	{
		glUniform1i(mShaderActive->get_uniform_location("objectId"),gameobject->mObjectId);	
	}
	
	else if (mShaderActive == mShaderOverlay) //overlay shader does not need lighting calculations and only needs orthographic projection
	{
		if (gameobject->mType == GameObject::Types::OverlayRectangle)
		{
			glUniformMatrix4fv(mShaderActive->get_uniform_location("mvpMatrix"), 1, GL_FALSE,  glm::value_ptr(glm::mat4())); //identity matrix for fullscreen quad
			glUniform2i(mShaderActive->get_uniform_location("recLowerLeft"), mPickBufferFrom.x, mPickBufferFrom.y); 
			glUniform2i(mShaderActive->get_uniform_location("recSize"), mPickBufferSize.x, mPickBufferSize.y);
			glUniform1i(mShaderActive->get_uniform_location("showPickBufferRectangle"), 1); 
			glUniform1i(mShaderActive->get_uniform_location("bubbleCursor"), 0); 
			
		}
		else
		{		
			glUniformMatrix4fv(mShaderActive->get_uniform_location("mvpMatrix"), 1, GL_FALSE,  glm::value_ptr(gameobject->getModelMatrix()));
			glUniform1i(mShaderActive->get_uniform_location("useTexture"), model->mUseTexture);
			glUniform1i(mShaderActive->get_uniform_location("bubbleCursor"), PickingHelper::getInstance()->m_showBubbleCursor); 
			glUniform2i(mShaderActive->get_uniform_location("mousePos"), mMousePosition->x, (mScreenHeight - mMousePosition->y)); 
			glUniform1f(mShaderActive->get_uniform_location("bubbleMaxDistance"), mDistance);	
			glUniform1i(mShaderActive->get_uniform_location("showPickBufferRectangle"), 0); 
		}

		

	}
	else  //normal blinnphong and skyboxshader uniforms
	{
		int useTexture = model->mUseTexture;
		if ((!mTransparency) && (model->mUseTexture == 2 || model->mUseTexture == 3)) { useTexture = 1; } 
		glUniformMatrix4fv(mShaderActive->get_uniform_location("mMatrix"), 1, GL_FALSE, glm::value_ptr(modelMatrix));
		glUniformMatrix4fv(mShaderActive->get_uniform_location("mvMatrix"), 1, GL_FALSE, glm::value_ptr(modelViewMatrix));
		glUniformMatrix3fv(mShaderActive->get_uniform_location("normalMatrix"), 1, GL_FALSE, glm::value_ptr(normalMatrix));
	
		glUniform4fv(mShaderActive->get_uniform_location("diffuseColor"), 1, glm::value_ptr(*model->mDiffuseColor));
		glUniform1f(mShaderActive->get_uniform_location("materialFactor"), model->mMaterialFactor);
		glUniform1i(mShaderActive->get_uniform_location("useTexture"), useTexture);
		glUniform1i(mShaderActive->get_uniform_location("useLight"),model->mUseLight);
		glUniform1f(mShaderActive->get_uniform_location("badFactor"), mBadFactor);
		float ambientFactor = 0.1f;
		if (gameobject->mHighlighted) { ambientFactor = 0.8f; }
		glUniform1f(mShaderActive->get_uniform_location("ambientFactor"), ambientFactor);
		glm::vec3 lightModifiedPosition = glm::vec3(0.0f,0.0f,0.0f);
		float lightFactor = mLight->mPosition->y / 50.0f;
		if (mNightMode && sector != NULL) 
		{ 
			lightFactor = 1; 
			lightModifiedPosition = *sector->mPosition; 
		}
		glUniform1f(mShaderActive->get_uniform_location("lightFactor"), glm::max(0.0f,glm::min(1.0f, lightFactor)));
		glm::vec3 vEyeLight = glm::vec3((*mCamera->mViewMatrix) * glm::vec4(*mLight->mPosition + lightModifiedPosition, 1));
		glUniform3fv(mShaderActive->get_uniform_location("vLightPosition"), 1, glm::value_ptr(vEyeLight));
		//if (model->mUseTexture == 5)
		//{
		//	glm::vec3 vRandom = glm::vec3(randFloat(0,1),randFloat(0,1),randFloat(0,1));
		//	glUniform3fv(shader->get_uniform_location("vRandom"), 1, glm::value_ptr(vRandom));
		//}
	}
}

void Scene::drawModel(const Model * model, const string mode, bool specialoffset, int specialoffsetposition)
{
	for (int i=0, offset=0;i<model->mNumberOfMeshes;i++)
	{
		if ( (mode == "normal") || (mode == "special" && i == specialoffsetposition && specialoffset) || ( mode == "special" && i!=specialoffsetposition && !specialoffset) ) { glDrawArrays(GL_TRIANGLES, offset, (model->mNumberOfVerticesPerMesh)[i]); } //15 is the bark
		offset+=(model->mNumberOfVerticesPerMesh)[i];
	}
}

void Scene::draw()
{	
	mCountDrawnObjects = 0;
	mCountCulledObjects = 0;
	//double timeBefore = glfwGetTime();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	//glEnable(GL_CULL_FACE);
	if (mWireFrame) {glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); }
	else { glPolygonMode(GL_FRONT_AND_BACK, GL_FILL); }

	// draw skybox  in VAO
	glDisable(GL_BLEND);
	setShaderActive(mShaderSkybox);
	glBindVertexArray(mModelSkybox->mVaoId);
	setUniforms(mModelSkybox, mSkybox);
	glDrawArrays(GL_TRIANGLES, 0, (sizeof(skybox_positions) / sizeof(float) / 4));

	if (mTransparency)
	{
		//enable alpha blending
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	else
	{
		glDisable(GL_BLEND);
	}
	setShaderActive(mShaderBlinnphong);

	// pass the texture unit to the shader
	glUniform1i(mShaderBlinnphong->get_uniform_location("colorTexture"), 1);
	glUniform1i(mShaderBlinnphong->get_uniform_location("alphaTexture"), 2);

	//pass the projection texture to the shader
	if (mLightning->mVisible)
	{
		glUniform1i(mShaderBlinnphong->get_uniform_location("projTexture"), 3);
		glActiveTexture(GL_TEXTURE0 + 3);
		if (mLightning->mType == GameObject::Types::PersonGood) {	glBindTexture(GL_TEXTURE_2D, mModelLightning->mTextureHandles[4]); }
		else													{	glBindTexture(GL_TEXTURE_2D, mModelLightning->mTextureHandles[5]); }

		float projDistanceFactor = int(10 + 400 * mLightningAlive) % int(10 + 400 * LIGHTNING_VISIBLE_SECONDS); //10 + 400 * 0.4
		glm::vec3 lightningPos = (*mLightning->mPosition) + (*mLightning->mDirection) * -1.0f * projDistanceFactor ;
		glm::mat4 modelViewMatrixTexProj = glm::lookAt(lightningPos, lightningPos + (*mLightning->mDirection),glm::vec3(0.0,+1.0,0.0));
		glm::mat4 modelViewProjectionMatrixTexProjAndCorrectment =  *mProjectionMatrixTexProj * modelViewMatrixTexProj;
		glUniformMatrix4fv(mShaderActive->get_uniform_location("texMatrix"), 1, GL_FALSE, glm::value_ptr(modelViewProjectionMatrixTexProjAndCorrectment));
	}
	else { glUniformMatrix4fv(mShaderActive->get_uniform_location("texMatrix"), 1, GL_FALSE, glm::value_ptr(glm::mat4())); }

	// draw ground
	glBindVertexArray(mModelGround->mVaoId);
	glActiveTexture(GL_TEXTURE0 + 1);
	glBindTexture(GL_TEXTURE_2D, mModelGround->mTextureHandles[0]);
	for (int i=0;i<FIELDSPERROW;i++) 
	{
		for (int j=0;j<FIELDSPERROW;j++)
		{
			if (checkFrustumCulling(this->mGroundParts[(i*FIELDSPERROW) + j]))
			{	
				setUniforms(mModelGround,mGroundParts[(i*FIELDSPERROW) + j]);
				glDrawArrays(GL_TRIANGLES, 0, (sizeof(ground_positions) / sizeof(float) / 4));
			}
		}
	}

	//draw ground borders
	glBindVertexArray(mModelGroundBorder->mVaoId);	
	glActiveTexture(GL_TEXTURE0 + 1);
	glBindTexture(GL_TEXTURE_2D, mModelGroundBorder->mTextureHandles[0]);
	for (int i=0;i<4;i++)
	{
		setUniforms(mModelGroundBorder,mGroundBorderParts[i]);
		drawModel(this->mModelGroundBorder);
	}

	//draw infopanel
	if (mInfoPanel->mVisible)
	{
		glBindVertexArray(mModelInfoPanel->mVaoId);
		glActiveTexture(GL_TEXTURE0 + 1);
		glBindTexture(GL_TEXTURE_2D, mModelInfoPanel->mTextureHandles[(int)this->mState]);
		setUniforms(mModelInfoPanel,mInfoPanel);
		glDrawArrays(GL_TRIANGLES, 0, (sizeof(ground_positions) / sizeof(float) / 4));
	}
	//draw light (only if not in nightmode)
	if (!mNightMode)
	{
		glBindVertexArray(mModelLight->mVaoId);
		//my_glBindVertexArray(mModelLight->mVaoId);
		setUniforms(mModelLight,mLight);	
		glDrawArrays(GL_TRIANGLES, 0, (sizeof(triangle_positions) / sizeof(float) / 4));
	}

	//draw sectorstonewalls
	glBindVertexArray(mModelStoneWall->mVaoId);						
	for (int i=0;i<SECTOR_COUNT;i++)  { mSectors[i]->drawStoneWall(); } 
	
	//draw moai heads
	glBindVertexArray(mModelMoai->mVaoId);		
	glActiveTexture(GL_TEXTURE0 + 1);
	glBindTexture(GL_TEXTURE_2D, mModelMoai->mTextureHandles[0]);
	for (int i=0;i<SECTOR_COUNT;i++)   { mSectors[i]->drawMoai(); } 
	
	//draw trees
	glBindVertexArray(mModelTree->mVaoId);	
	
	//draw treeleaves
	glUniform1i(mShaderBlinnphong->get_uniform_location("useTexture"),2);						
	glActiveTexture(GL_TEXTURE0 + 1);
	glBindTexture(GL_TEXTURE_2D, mModelTree->mTextureHandles[0]);
	glActiveTexture(GL_TEXTURE0 + 2);
	glBindTexture(GL_TEXTURE_2D, mModelTree->mTextureHandles[1]);	
	for (int i=0;i<SECTOR_COUNT;i++)   { mSectors[i]->drawTree(false); } 
	
	//draw barks
	glUniform1i(mShaderBlinnphong->get_uniform_location("useTexture"),1);
	glActiveTexture(GL_TEXTURE0 + 1);
	glBindTexture(GL_TEXTURE_2D, mModelTree->mTextureHandles[2]);					
	for (int i=0;i<SECTOR_COUNT;i++)   { mSectors[i]->drawTree(true); } 
	
	//draw holygrenades
	glBindVertexArray(mModelHolyGrenade->mVaoId);
	glActiveTexture(GL_TEXTURE0 + 1);
	glBindTexture(GL_TEXTURE_2D, mModelHolyGrenade->mTextureHandles[0]);				
	for (int i=0;i<SECTOR_COUNT;i++)  { mSectors[i]->drawHolyGrenade(); } 
	
	//draw holygrenadesatellites
	glActiveTexture(GL_TEXTURE0 + 1);
	glBindTexture(GL_TEXTURE_2D, mModelHolyGrenade->mTextureHandles[1]);				
	for (int i=0;i<SECTOR_COUNT;i++)  { mSectors[i]->drawHolyGrenadeSatellite(); } 	
	
	//draw holygrenadeborders
	glBindVertexArray(mModelHolyGrenadeBorder->mVaoId);
//	glActiveTexture(GL_TEXTURE0 + 1);
//	glBindTexture(GL_TEXTURE_2D, mModelHolyGrenade->mTextureHandles[0]);			
	for (int i=0;i<SECTOR_COUNT;i++)   { mSectors[i]->drawHolyGrenadeBorder(); } 
	
	//draw peoplegood
	glBindVertexArray(mModelPeopleGood->mVaoId);
	glActiveTexture(GL_TEXTURE0 + 1);
	glBindTexture(GL_TEXTURE_2D, mModelPeopleGood->mTextureHandles[0]);
	for (int i=0;i<SECTOR_COUNT;i++)   { mSectors[i]->drawPeopleGood(); } 
	
	//draw peoplebad
	glBindVertexArray(mModelPeopleBad->mVaoId);
	glActiveTexture(GL_TEXTURE0 + 1);
	glBindTexture(GL_TEXTURE_2D, mModelPeopleBad->mTextureHandles[0]);
	for (int i=0;i<SECTOR_COUNT;i++)   { mSectors[i]->drawPeopleBad(); } 
	
	//draw lightning 
	if (mLightning->mVisible)
	{
		int textureoffset = 0;
		if (mLightning->mType == GameObject::Types::PersonGood) { textureoffset = 2; }
		//glDisable(GL_DEPTH_TEST);		
		glBindVertexArray(mModelLightning->mVaoId);
		setUniforms(mModelLightning,mLightning);
		glActiveTexture(GL_TEXTURE0 + 1);
		glBindTexture(GL_TEXTURE_2D, mModelLightning->mTextureHandles[0 + textureoffset]);
		glActiveTexture(GL_TEXTURE0 + 2);
		glBindTexture(GL_TEXTURE_2D, mModelLightning->mTextureHandles[1 + textureoffset]);	
		glDrawArrays(GL_TRIANGLES, 0, (sizeof(ground_positions) / sizeof(float) / 4));
		//glEnable(GL_DEPTH_TEST);
	}

	//draw 2D Overlay Objects
	setShaderActive(mShaderOverlay);
	//set texture units
	glUniform1i(mShaderOverlay->get_uniform_location("colorTexture"), 1);
	glUniform1i(mShaderOverlay->get_uniform_location("alphaTexture"), 2);	
	
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	if (mMousePointer->mVisible)
	{
		if (!(PickingHelper::getInstance()->m_showBubbleCursor && mSelectedObject == nullptr))
		{
			glBindVertexArray(mModelOverlay->mVaoId);
			//draw mouse pointer
			glActiveTexture(GL_TEXTURE0 + 1);
			if (!PickingHelper::getInstance()->m_showBubbleCursor) { glBindTexture(GL_TEXTURE_2D, mModelOverlay->mTextureHandles[0]); }
			setUniforms(mModelOverlay,mMousePointer);
			glDrawArrays(GL_TRIANGLES, 0, (sizeof(overlay_positions) / sizeof(float) / 4));	
		}
	}
	if (PickingHelper::getInstance()->m_showPickbufferRectangle)
	{
		mMousePointer->mType = GameObject::Types::OverlayRectangle; //misuse the mouse-pointer-object as rectangle
		setUniforms(mModelOverlay,mMousePointer);
		glDrawArrays(GL_TRIANGLES, 0, (sizeof(overlay_positions) / sizeof(float) / 4));	
		mMousePointer->mType = GameObject::Types::Overlay;
	}
	
	glEnable(GL_DEPTH_TEST);
	// Unbind VBOs
	glBindVertexArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	drawObjectIDs();
}
	


void Scene::drawObjectIDs()
{
	//draw fbo things for objectpicking
	setShaderActive(mShaderFbo);
	// bind fbo
	glBindFramebuffer(GL_FRAMEBUFFER, mFboId);
	glViewport(0, 0, mScreenWidth, mScreenHeight);
	// clear our color- and depth-buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDisable(GL_BLEND);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	//holygrenade
	glBindVertexArray(mModelHolyGrenade->mVaoId);	
	for (int i=0;i<SECTOR_COUNT;i++)   { if (mSectors[i]->mHolyGrenade->mState == GameObject::States::CraftingFinished) { mSectors[i]->drawHolyGrenade(); } }
	//tree
	glBindVertexArray(mModelTree->mVaoId);	
	for (int i=0;i<SECTOR_COUNT;i++)  { mSectors[i]->drawTree(true); mSectors[i]->drawTree(false); } 
	//draw peoplegood
	glBindVertexArray(mModelPeopleGood->mVaoId);
	for (int i=0;i<SECTOR_COUNT;i++)   { mSectors[i]->drawPeopleGood(); } 
	//draw peoplebad
	glBindVertexArray(mModelPeopleBad->mVaoId);
	for (int i=0;i<SECTOR_COUNT;i++)  { mSectors[i]->drawPeopleBad(); } 
	glEnable(GL_BLEND);
	// unbind fbo
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
	//double timeAfter = glfwGetTime();				
	//cout << "\r\nElapsed Time: (s) " << (timeAfter - timeBefore)*1000.0 << endl;
}
