#pragma once
#pragma warning( disable : 4482 )

#include "Shader.hpp"
#include "Camera.hpp"
#include <map>
#include <set>
#include "SoundInterface.hpp"
#include <vector>
#include "Plane.hpp"

class Sector;
class GameObject;
class Model;
struct ViewPlane;

/**
* This Class Contains most of the Game Logic and Draw Calls.
* The methods relevant for semantic pointing are updateCursorPosition and updateObjectPicking.
*/
class Scene
{
public:
	static enum States { Normal, Won, Lost, Init };	
	Scene(void);
	~Scene(void);
	bool Scene::init(void);
	void Scene::update(float timeDiff);

	/**
	* This method updates the position of our own visual mouse cursor according to the speed which is dependent on the distance to the nearest object.
	* it uses mCursorFactor to calculate the speed
	*/	
	void Scene::updateCursorPosition();
	void Scene::updateBorderParts(float timeDiff);
	void Scene::updateCamera(float timeDiff, bool firstUpdate);
	void Scene::updateLight(float timeDiff);
	void Scene::updateLightning(float timeDiff);
	void Scene::updateSkyBox(float timeDiff);
	void Scene::updateInfoPanel();
	
	/**
	* Reads the Pickbuffer (or at least a part of it) from the GPU memory and calculates the nearest object.
	* It stores the nearest found object in mSelectedObject
	* @param timeDiff is unused
	*/
	void Scene::updateObjectPicking(float timeDiff);
	void Scene::updateSectors(float timeDiff, bool firstUpdate);
	void Scene::updateBadFactor(float timeDiff);
	void Scene::updateSound(float timeDiff);

	void Scene::draw(void);
	
	/**
	* Draws the pickable objects into a FBO with objectIds as pixel values 
	*/
	void Scene::drawObjectIDs();
	void Scene::handleShoot(int button);
	bool Scene::checkFrustumCulling(GameObject *gameObject);
	void Scene::resize(int x, int y);
	void Scene::toggleMipMapping(void);
	void Scene::toggleTrilinear(void);
	void Scene::toggleWireFrame(void);
	void Scene::toggleTransparency(void);
	void Scene::toggleFrustumCulling(void);
	void Scene::toggleNightMode(void);
	void Scene::togglePause(glm::vec3 *cameraPosition = NULL);
	void Scene::generateRandom();
	void Scene::setUniforms(const Model *model, const GameObject *gameobject, const Sector *sector = NULL);
	void Scene::drawModel(const Model * model,const string mode = "normal", bool specialoffset = false, int specialoffsetposition = -1);
	void Scene::playSingleSoundDistance(float maxLength, glm::vec3 positionObject, SoundInterface::Sounds soundType, bool debugOutput = false);
	States mState;
	int mScreenWidth, mScreenHeight;
	Model *mModelTree, *mModelMoai, *mModelGround, *mModelGroundBorder, *mModelInfoPanel, *mModelSkybox, *mModelPeopleGood, *mModelPeopleBad, *mModelLight, *mModelHolyGrenade, *mModelHolyGrenadeBorder, *mModelLightning, *mModelStoneWall, *mModelOverlay;
	Camera *mCamera;
	GameObject *mLight, *mLightning, *mSkybox, **mGroundParts, **mGroundBorderParts, *mInfoPanel;
	GameObject *mMousePointer;
	glm::mat4 *mProjectionMatrix, *mProjectionMatrixTexProj, *mProjectionMatrixOrtho;
	Shader *mShaderBlinnphong, *mShaderSkybox, *mShaderFbo, *mShaderOverlay;
	float mBadFactor, mLightningAlive, mSpeedFactor, mTimeDiffWonOrLostWait;
	int mBadSwapping, mCountDrawnObjects, mCountCulledObjects;
	bool mNightMode, mPaused, mMipMapping, mTrilinear, mWireFrame, mTransparency, mFrustumCulling;
	Sector **mSectors;
	SoundInterface *mSoundInterface;
	glm::ivec2 *mMousePosition;
	Plane mPlanes[6];
	static std::map<int, GameObject*> sGameObjects;

private:
	bool mLightningVaoOnlyUpdate;
	GLuint mFboId, mFboImgId;
	Shader *mShaderActive;
	void Scene::setShaderActive(Shader *shader);
	glm::ivec2 *mCursorLastPos;
	double mCursorFactor, mDistance;
	GameObject* mSelectedObject;
	glm::ivec2 mPickBufferFrom, mPickBufferSize;
};

