#include "Sector.hpp"
#include "GameObject.hpp"
#include "Scene.hpp"
#include "Model.hpp"
#include "utility.hpp"
#include "SoundInterface.hpp"

Sector::Sector(void) 
{
}
Sector::Sector(glm::vec3 *position, int sectornumber,  Scene *scene)
{
	this->mPosition = position;
	this->mSectorNumber = sectornumber;
	this->mScene = scene;
}

Sector::~Sector(void)
{
}

bool Sector::init(void)
{
	this->mState = Sector::States::Normal;
	this->mTree = new GameObject(GameObject::Types::Tree, this);
	*this->mTree->mScale = glm::vec3(12.0f, 12.0f,12.0f); //default 8
	this->mTree->mRotationMax->x = this->mTree->mRotationMax->z = TREE_ROTATION_SPEED_MAXIMUM;
	this->mMoai = new GameObject(GameObject::Types::Moai, this);
	*this->mMoai->mScale = glm::vec3(0.4f,0.4f,0.4f);	
	this->mHolyGrenade = new GameObject(GameObject::Types::HolyGrenade, this);
	mHolyGrenadeScaleDefault = 30.0f;
	float mHolyGrenadeScaleBorderDefault = mHolyGrenadeScaleDefault / 200.0f;
	*this->mHolyGrenade->mScale = glm::vec3(mHolyGrenadeScaleDefault, mHolyGrenadeScaleDefault, mHolyGrenadeScaleDefault);
	*this->mHolyGrenade->mRotationSpeed = glm::vec3(0.0f,0.0f,0.0f);
	this->mHolyGrenade->mRotationMax->x = this->mHolyGrenade->mRotationMax->z = 0;
	this->mHolyGrenade->mState = GameObject::States::Crafting;
	this->mHolyGrenadeBorder = new GameObject(GameObject::Types::Other, this);
	*this->mHolyGrenadeBorder->mScale = glm::vec3(mHolyGrenadeScaleBorderDefault, mHolyGrenadeScaleBorderDefault, mHolyGrenadeScaleBorderDefault);
	this->mHolyGrenadeSatellite = new GameObject(GameObject::Types::Other, this);
	*this->mHolyGrenadeSatellite->mScale = *this->mHolyGrenade->mScale / 7.0f;
	this->mHolyGrenadeBorder->mState = GameObject::States::Crafting;	
	this->mStoneWall = new GameObject(GameObject::Types::Other, this);
	*this->mStoneWall->mPosition = *this->mPosition;
	float stonewallscale = SECTOR_WIDTH_MINUS_MARGIN_PEOPLE / 10.0f; //default: 13.8f
	*this->mStoneWall->mScale = glm::vec3(stonewallscale * 1.08f,stonewallscale / 1.0f,stonewallscale * 1.08f);
	this->mWonLostStatue = new GameObject(GameObject::Types::Other, this);
	float wonloststatuescale = 75.0f;
	*this->mWonLostStatue->mPosition = glm::vec3(this->mPosition->x,-wonloststatuescale * 3,this->mPosition->z);
	*this->mWonLostStatue->mScale = glm::vec3(wonloststatuescale,wonloststatuescale,wonloststatuescale);
	this->mWonLostStatue->mState = GameObject::States::Moving;
	this->mTimeDiffGoodsSpawnTime = 0;
	this->mTimeDiffStateWarning = 0;
	this->mCountCraftingPeople = 0;
	this->mCountShotPeopleGood = 0;
	this->mCountCraftedPoints = 0;
	this->mDiffuseColor = new glm::vec4(1.0f,1.0f,1.0f,1.0f);
	this->mBadState = 0;
	this->mPeopleGood = new std::vector<GameObject *>();	
	this->mPeopleBad = new std::vector<GameObject *>();
	
	generateRandom();
	return true;
}
//generate random positions for tree, moai and handgrenade
void Sector::generateRandom(void)
{
	mGoodsSpawnTime = randFloat(GOODS_SPAWNTIME_MINIMUM_PERSECOND, GOODS_SPAWNTIME_MAXIMUM_PERSECOND);
	do {
		*mTree->mPosition = *this->mPosition + glm::vec3((GLfloat)(rand()%SECTOR_WIDTH_MINUS_MARGIN_BIGOBJECTS - SECTOR_WIDTH_MINUS_MARGIN_BIGOBJECTS/2), 0, (GLfloat)(rand()%SECTOR_LENGTH_MINUS_MARGIN_BIGOBJECTS - SECTOR_LENGTH_MINUS_MARGIN_BIGOBJECTS/2));
	} 
	while (false);
	do {
		*mMoai->mPosition = *this->mPosition + glm::vec3((GLfloat)(rand()%SECTOR_WIDTH_MINUS_MARGIN_BIGOBJECTS - SECTOR_WIDTH_MINUS_MARGIN_BIGOBJECTS/2), 0, (GLfloat)(rand()%SECTOR_LENGTH_MINUS_MARGIN_BIGOBJECTS - SECTOR_LENGTH_MINUS_MARGIN_BIGOBJECTS/2));
	}
	while (glm::length(*mTree->mPosition - *mMoai->mPosition) < SECTOR_WIDTH_MINUS_MARGIN_BIGOBJECTS/2);
	do {
		*mHolyGrenade->mPosition = *this->mPosition + glm::vec3((GLfloat)(rand()%SECTOR_WIDTH_MINUS_MARGIN_BIGOBJECTS - SECTOR_WIDTH_MINUS_MARGIN_BIGOBJECTS/2), 0, (GLfloat)(rand()%SECTOR_LENGTH_MINUS_MARGIN_BIGOBJECTS - SECTOR_LENGTH_MINUS_MARGIN_BIGOBJECTS/2));
	}
	while (glm::length(mHolyGrenade->getPositionXZ() - *mMoai->mPosition) < SECTOR_WIDTH_MINUS_MARGIN_BIGOBJECTS/2 || glm::length(mHolyGrenade->getPositionXZ() - *mTree->mPosition) < SECTOR_WIDTH_MINUS_MARGIN_BIGOBJECTS/2);	

	mMoai->mRotationSpeed->y = randFloat(MOAI_ROTATION_SPEED_MINIMUM, MOAI_ROTATION_SPEED_MAXIMUM);
	mTree->mRotationSpeed->x = randFloat(TREE_ROTATION_SPEED_MINIMUM, TREE_ROTATION_SPEED_MAXIMUM);
	mTree->mRotationSpeed->z = randFloat(TREE_ROTATION_SPEED_MINIMUM, TREE_ROTATION_SPEED_MAXIMUM);
	*mMoai->mRotation = glm::vec3(0.0f,0.0f,0.0f);
	*mTree->mRotation = glm::vec3(0.0f,0.0f,0.0f);
	*mHolyGrenadeBorder->mPosition = *mHolyGrenade->mPosition;
}

void Sector::update(float timeDiff, bool firstUpdate)
{
	if (mScene->mState != Scene::States::Init || firstUpdate)
	{
		updateHolyGrenade(timeDiff);
		updateHolyGrenadeSatellite(timeDiff);
		updateMoai(timeDiff);
		updateTree(timeDiff);
		updatePeopleGood(timeDiff);
		updatePeopleBad(timeDiff);
		updateStoneWall(timeDiff);
	}
	//sector himself the one and only
	if (mPeopleBad->size() > mPeopleGood->size() || mState == Sector::States::Warning || mState == Sector::States::Lost)	{ mBadState = 1; *mDiffuseColor = glm::vec4(1.0f,0.5f,0.5f,1.0f); }
	else											{ mBadState = -1; *mDiffuseColor = glm::vec4(0.3f,0.3f,1.0f,1.0f); }
}

void Sector::updateHolyGrenade(float timeDiff)
{					
	if (this->mHolyGrenade->mState == GameObject::States::Crafting)
	{
		if (this->mCountCraftingPeople > 0) {	this->mCountCraftedPoints += this->mCountCraftingPeople * HOLYGRENADE_CRAFTING_POINTS_PERSECOND * timeDiff; }
		else  {									this->mCountCraftedPoints += (float)HOLYGRENADE_NOCRAFTING_POINTS_PERSECOND * timeDiff;  }
		
		if (this->mCountCraftedPoints >= HOLYGRENADE_CRAFTING_POINTSTOWIN) //crafting finished
		{
			this->mHolyGrenade->mState = GameObject::States::CraftingFinished;
			//this->mHolyGrenade->mPosition->y = -0.95f  * mHolyGrenade->mScale->y + 1.87f * mHolyGrenade->mScale->y;
			this->mHolyGrenade->mRotation->x = 0;
			this->mHolyGrenade->mRotation->z = 0;
			mHolyGrenade->setDirectionXZ(*mTree->mPosition - mHolyGrenade->getPositionXZ());
			mScene->mSoundInterface->play(SoundInterface::Sounds::SoundHolyGrenadeFinished);
			mHolyGrenadeBorder->mState = GameObject::States::Dead;
		}
		else if (this->mCountCraftedPoints <= HOLYGRENADE_NOCRAFTING_POINTSTOLOSE)  //you lost
		{
			this->mState = Sector::States::Lost;
			this->mHolyGrenade->mState = GameObject::States::CraftingLost;
			this->mHolyGrenade->mRotation->x = 0;
			this->mHolyGrenade->mRotation->z = 0;
		}
		else //between win and lose
		{
			this->mState = Sector::States::Normal;
			if (this->mCountCraftedPoints >= 0)
			{
				this->mHolyGrenade->mPosition->y = -0.95f  * mHolyGrenade->mScale->y + 1.87f * mHolyGrenade->mScale->y * this->mCountCraftedPoints / HOLYGRENADE_CRAFTING_POINTSTOWIN;  
				this->mHolyGrenade->mRotationMax->x = (float)glm::min(this->mCountCraftingPeople, 30);
				this->mHolyGrenade->mRotationMax->z = (float)glm::min(this->mCountCraftingPeople, 30);
	
				this->mHolyGrenade->mRotationSpeed->x = this->mCountCraftingPeople  / 1.0f * sign(this->mHolyGrenade->mRotationSpeed->x); 
				this->mHolyGrenade->mRotationSpeed->z = this->mCountCraftingPeople / 1.5f * sign(this->mHolyGrenade->mRotationSpeed->z);
				this->mHolyGrenade->mRotationSpeed->y = this->mCountCraftingPeople / 0.5f;
				calculateRotation(this->mHolyGrenade, timeDiff);
			}
			else
			{
				if (this->mCountCraftedPoints <= HOLYGRENADE_NOCRAFTING_POINTSTOLOSE * 0.7f) //you get warned that you are losing
				{
					this->mState = Sector::States::Warning;
				}
				//cout << this->mCountCraftedPoints << " " << HOLYGRENADE_NOCRAFTING_POINTSTOLOSE << endl;
				this->mHolyGrenade->mPosition->y = -0.95f  * mHolyGrenade->mScale->y - 0.8f * mHolyGrenade->mScale->y * this->mCountCraftedPoints / HOLYGRENADE_NOCRAFTING_POINTSTOLOSE;  
			}

		}
	}
	else if (this->mHolyGrenade->mState == GameObject::States::MovingToTree)
	{
		*mHolyGrenade->mPosition += mHolyGrenade->getDirectionXZ() * timeDiff * HOLYGRENADE_MOVE_PERSECOND;
		mHolyGrenade->mRotationSpeed->x  = -HOLYGRENADE_ROTATION_SPEED;
		calculateRotation(this->mHolyGrenade, timeDiff);
		//collision with tree
		if (glm::length(*mTree->mPosition - mHolyGrenade->getPositionXZ()) < TREE_RADIUS_COLLISION) 
		{ 
			mHolyGrenade->mState = GameObject::States::MovingToTreeFinished;
			mTree->mState = GameObject::States::Dead;
			this->mState = Sector::States::Won;
			mScene->mSoundInterface->play(SoundInterface::Sounds::SoundHolyGrenadeExplosion);
		}
	} 
	else if (this->mHolyGrenade->mState == GameObject::States::MovingToTreeFinished) //you won
	{	
		mHolyGrenade->mRotation->x = 0.0f;	
		*mHolyGrenade->mScale *= (1.0f + (2.0f * timeDiff));
		mHolyGrenade->mPosition->y *= (1.0f + (2.0f * timeDiff));
		if (mHolyGrenade->mScale->x >= 15 * mHolyGrenadeScaleDefault) 
		{  
			this->mHolyGrenade->mState = GameObject::States::Dead;	 
			mScene->mSoundInterface->play(SoundInterface::Sounds::SoundHolyGrenadeFinished);
		}
	}
}

void Sector::updateHolyGrenadeSatellite(float timeDiff)
{
	//holy grenade satellite
	*this->mHolyGrenadeSatellite->mPosition = *this->mHolyGrenade->mPosition;
	//this->mHolyGrenadeSatellite->mPosition->y += 100;
	//	*mCamera->mDirection = (glm::mat3(glm::rotate(-timeDiff * 20,0.0f,1.0f,0.0f)) * (*mCamera->mDirection));
	//*mCamera->mPosition =  (glm::length(*mCamera->mPosition / (1+ timeDiff/10)) * (*mCamera->mDirection) * -1.0f);
	mHolyGrenadeSatellite->mRotation->y += timeDiff* 200;
	*mHolyGrenadeSatellite->mPosition += mHolyGrenadeSatellite->getDirectionXZ() * 40.0f; 
}

void Sector::updateMoai(float timeDiff)
{
	this->mTimeDiffGoodsSpawnTime += timeDiff;	
	if (this->mTimeDiffGoodsSpawnTime >= this->mGoodsSpawnTime)
	{
		this->mTimeDiffGoodsSpawnTime = 0.0f;
		if (this->mPeopleGood->size() + this->mPeopleBad->size() < PEOPLE_COUNT_PERSECTOR_MAXIMUM - this->mCountShotPeopleGood)
		{
			GameObject *goodNew = new GameObject(GameObject::Types::PersonGood, this);
			goodNew->mType = GameObject::Types::PersonGood;
			goodNew->mState = GameObject::States::Moving;
			*goodNew->mRotation = *this->mMoai->mRotation;
			*goodNew->mPosition  = *this->mMoai->mPosition + goodNew->getDirectionXZ() * 5.0f;
			*goodNew->mScale = glm::vec3(7.5f, 7.5f,7.5f); //default: 3.0f
			goodNew->mRotationSpeed->x = randFloat(20.0f, 40.0f) * -1.0f;
			goodNew->mRotationSpeed->z = randFloat(120.0f, 140.0f) * -1.0f;			
			this->mPeopleGood->push_back(goodNew);
		}
	}
	calculateRotation(this->mMoai, timeDiff);
}
void Sector::updateTree(float timeDiff)
{
	calculateRotation(mTree, timeDiff);
	if (Sector::mState == Sector::States::Lost) { mTree->mPosition->y = this->mPosition->y; }
}
void Sector::updatePeopleGood(float timeDiff)
{
	float peoplemove = timeDiff * PEOPLE_MOVE_PERSECOND;
	std::vector<GameObject *>::iterator iter = mPeopleGood->begin();
	while(iter != mPeopleGood->end())
	{	
		GameObject *item = *iter;
		if (item->mState == GameObject::States::Dead) 
		{
			iter = this->mPeopleGood->erase(iter); 
			//delete item;
			continue;
		}
		else if (Sector::mState == Sector::States::Lost)
		{
			//same as when collide with tree, should be outsourced in own method
			item->mState = GameObject::States::Moving;
			item->mType = GameObject::Types::PersonBad;
			this->mPeopleBad->push_back(item);
			iter = this->mPeopleGood->erase(iter);
			continue;
		}
		else if (Sector::mState == Sector::States::Won)
		{
			item->mPosition->y = this->mPosition->y;
		}
		else if (item->mState == GameObject::States::Dying)
		{
			item->mTimeDiffDyingTime += timeDiff;
			if (item->mTimeDiffDyingTime >= PEOPLE_DYING_TIME)
			{
				this->mCountShotPeopleGood++;
				item->mState = GameObject::States::Dead;
				mScene->mSoundInterface->play(SoundInterface::Sounds::SoundGodRegret);
			}	
		}
		else if (item->mState == GameObject::States::Moving || item->mState == GameObject::States::BeingHijacked || item->mState == GameObject::States::MovingToCraft)
		{
			//collision detection with sector boundaries
			this->collisionWithSectorAndMove(item, peoplemove);
			if (item->mState == GameObject::States::Moving || item->mState == GameObject::States::BeingHijacked)
				{
				//suck-in from tree
				if (this->mTree->mState != GameObject::States::Dead && glm::length(*mTree->mPosition - *item->mPosition) < TREE_RADIUS_SUCKIN) 
				{ 
					item->setDirectionXZ(*mTree->mPosition - *item->mPosition);
				}
				//collision with tree
				if (this->mTree->mState != GameObject::States::Dead && glm::length(*mTree->mPosition - *item->mPosition) < TREE_RADIUS_COLLISION)  
				{ 
					item->mState = GameObject::States::Moving;
					item->mType = GameObject::Types::PersonBad;
					this->mPeopleBad->push_back(item);
					iter = this->mPeopleGood->erase(iter);
					
					//1st approach: sound which is always present: maxLength, SoundType, 
					//return the nearest distance
					
					//2nd approach: direct sound play: maxLength, SoundType, PositionObject, debugoutput=false
					mScene->playSingleSoundDistance(300.0f, *this->mTree->mPosition, SoundInterface::Sounds::SoundApple);
					continue;
				}
			}
			if (item->mState == GameObject::States::Moving || item->mState == GameObject::States::MovingToCraft)
			{
				//suck-in from holygrenade
				if (glm::length(mHolyGrenade->getPositionXZ() - *item->mPosition) < HOLYGRENADE_RADIUS_SUCKIN && mHolyGrenade->mState == GameObject::States::Crafting) 
				{ 
					item->setDirectionXZ(mHolyGrenade->getPositionXZ() - *item->mPosition);
				}
				//collision with holygrenade
				if (glm::length(mHolyGrenade->getPositionXZ() - *item->mPosition) < HOLYGRENADE_RADIUS_COLLISION  && mHolyGrenade->mState == GameObject::States::Crafting) 
				{ 
					item->mState = GameObject::States::Crafting;
					this->mCountCraftingPeople++;
					//cout << "CountCrafting increased: SectorNumber: " << this->mSectorNumber << ", Count: "<< mCountCraftingPeople << endl;
				}
			}
			else if (item->mState == GameObject::States::BeingHijacked)
			{
				//check if not being hijacked anymore
				if (item->mParent->mState == GameObject::States::Dying) 
				{ 
					item->mParent = NULL;
					item->mState = GameObject::States::MovingToCraft;
					item->mRotation->y += 180.0f; // go back to grenade

				}
			}
		}
		else if (item->mState == GameObject::States::Crafting)
		{
			if(mHolyGrenade->mState != GameObject::States::Crafting) //grenade is not crafting anymore, you can stop with the work
			{
				item->mState = GameObject::States::Moving;
			}
		}
		calculateRotation(item, timeDiff);
		++iter;
	}
}

void Sector::updatePeopleBad(float timeDiff)
{
	float peoplemove = timeDiff * PEOPLE_MOVE_PERSECOND;
	std::vector<GameObject *>::iterator iter = mPeopleBad->begin();
	while(iter != mPeopleBad->end())
	{
		GameObject *item = *iter;
		if (item->mState == GameObject::States::Dead) 
		{
			iter = this->mPeopleBad->erase(iter); 
			//delete item;
			continue;
		}
		else if (Sector::mState == Sector::States::Lost)
		{
			item->mPosition->y = this->mPosition->y;
		}
		else if (mTree->mState == GameObject::States::Dead)
		{
			item->mState = GameObject::States::Dead;
		}
		else if (item->mState == GameObject::States::Dying)
		{
			item->mTimeDiffDyingTime += timeDiff;
			if (item->mTimeDiffDyingTime >= PEOPLE_DYING_TIME)
			{
				item->mState = GameObject::States::Dead;
				mScene->mSoundInterface->play((SoundInterface::Sounds)randInt(SoundInterface::Sounds::SoundGodLaugh1, SoundInterface::Sounds::SoundGodLaugh4));
			}	
		}
		else if (item->mState == GameObject::States::Moving)
		{
			//collision detection with sector boundaries
			this->collisionWithSectorAndMove(item, peoplemove);
			//suck-in from holygrenadse
			if (this->mCountCraftingPeople > 0 && glm::length(mHolyGrenade->getPositionXZ() - *item->mPosition) < HOLYGRENADE_RADIUS_SUCKIN  && mHolyGrenade->mState == GameObject::States::Crafting) //only suck in if somebody is at the stone crafting
			{ 
				item->setDirectionXZ(mHolyGrenade->getPositionXZ() - *item->mPosition); 
			}
			//collision with holygrenade
			if (this->mCountCraftingPeople > 0 && glm::length(mHolyGrenade->getPositionXZ() - *item->mPosition) < HOLYGRENADE_RADIUS_COLLISION  && mHolyGrenade->mState == GameObject::States::Crafting) //only collide if somebody is at the stone crafting
			{ 
				for (int i=0;i<(int)mPeopleGood->size();i++) 
				{ 
					if (mPeopleGood->at(i)->mState == GameObject::States::Crafting) 
					{
						mCountCraftingPeople--;
						//cout << "CountCrafting decreased: SectorNumber: " << this->mSectorNumber << ", Count: "<< mCountCraftingPeople << endl;
						item->mState = GameObject::States::Hijacking;
						mPeopleGood->at(i)->mState = GameObject::States::BeingHijacked;
						mPeopleGood->at(i)->mParent = item;
						item->setDirectionXZ(*mTree->mPosition - *item->mPosition);
						mPeopleGood->at(i)->setDirectionXZ(*mTree->mPosition - *item->mPosition);						
						*mPeopleGood->at(i)->mPosition = *item->mPosition - mPeopleGood->at(i)->getDirectionXZ() * 10.0f;
						break;
					}
				}
			}
		}
		else if (item->mState == GameObject::States::Hijacking)
		{
			//collision detection with sector boundaries
			this->collisionWithSectorAndMove(item, peoplemove);
			//collision with tree
			if (glm::length(*mTree->mPosition - *item->mPosition) < TREE_RADIUS_COLLISION) 
			{ 
				item->mState = GameObject::States::Moving;
			}
		}	
		calculateRotation(item, timeDiff);
		++iter;
	}
}

void Sector::updateStoneWall(float timeDiff)
{
	if (mState == Sector::States::Warning)
	{
		mTimeDiffStateWarning += timeDiff;
		if (mTimeDiffStateWarning >= 0.7f)
		{
			if (mStoneWall->mToggleSwitch) { mScene->mSoundInterface->play(SoundInterface::Sounds::SoundWarning); }
			mTimeDiffStateWarning = 0;
			mStoneWall->mToggleSwitch = !mStoneWall->mToggleSwitch;
		}
		mStoneWall->mHighlighted = mStoneWall->mToggleSwitch;
		this->mHolyGrenadeBorder->mHighlighted = mStoneWall->mToggleSwitch;
	}

	if ((mState == Sector::States::Won || mState == Sector::States::Lost) && mWonLostStatue->mState != GameObject::States::CraftingFinished) 
	{
		mStoneWall->mHighlighted = false;
		mStoneWall->mPosition->y += timeDiff * 15;
		mWonLostStatue->mPosition->y = - 3 * mWonLostStatue->mScale->y * ( 1.0f - mStoneWall->mPosition->y / 60.0f);
		if (mStoneWall->mPosition->y >= 60.0f) { mWonLostStatue->mState = GameObject::States::CraftingFinished; }
	}
	if (mState == Sector::States::Won && mWonLostStatue->mState == GameObject::States::CraftingFinished)
	{
		mStoneWall->mPosition->y += timeDiff * 30;
		*this->mPosition = *mStoneWall->mPosition - 60.0f;
	}
	if (mState == Sector::States::Lost && mWonLostStatue->mState == GameObject::States::CraftingFinished)
	{
		mStoneWall->mPosition->y -= timeDiff * 3;
		*this->mPosition = *mStoneWall->mPosition - 60.0f;
	}
}
void Sector::collisionWithSectorAndMove(GameObject *item, float peoplemove)
{
		if		(item->mPosition->x >= this->mPosition->x + SECTOR_WIDTH_MINUS_MARGIN_PEOPLE/2)   { do {item->mRotation->y += randFloat(70,130); } while (item->getDirectionXZ().x >= -0.01f); }
		else if (item->mPosition->x <= this->mPosition->x - SECTOR_WIDTH_MINUS_MARGIN_PEOPLE/2)   { do {item->mRotation->y += randFloat(70,130); } while (item->getDirectionXZ().x <= +0.01f); }
		else if (item->mPosition->z <= this->mPosition->z - SECTOR_LENGTH_MINUS_MARGIN_PEOPLE/2)  { do {item->mRotation->y += randFloat(70,130); } while (item->getDirectionXZ().z <= +0.01f); }
		else if (item->mPosition->z >= this->mPosition->z + SECTOR_LENGTH_MINUS_MARGIN_PEOPLE/2)  { do {item->mRotation->y += randFloat(70,130); } while (item->getDirectionXZ().z >= -0.01f); }
		*item->mPosition += item->getDirectionXZ() * peoplemove;
}
void Sector::calculateRotation(GameObject *item, float timeDiff)
{
	if (item->mType == GameObject::Types::PersonGood || item->mType == GameObject::Types::PersonBad)
	{
		if (item->mState == GameObject::States::Moving || item->mState == GameObject::States::Hijacking || item->mState == GameObject::States::MovingToCraft)
		{
			//rotation z
			if (item->mRotation->z <= -30.0f) { item->mRotationSpeed->z = abs(item->mRotationSpeed->z); }
			if (item->mRotation->z >= +30.0f) { item->mRotationSpeed->z = abs(item->mRotationSpeed->z) * -1.0f; }
			item->mRotation->z += item->mRotationSpeed->z * timeDiff;
			item->mRotation->x = 0.0f;

		}
		else if (item->mState == GameObject::States::Crafting)
		{
			//rotation x
			if (item->mRotation->x <= -30.0f) { item->mRotationSpeed->x = abs(item->mRotationSpeed->x); }
			if (item->mRotation->x >=   0.0f) { item->mRotationSpeed->x = abs(item->mRotationSpeed->x) * -1.0f; }
			item->mRotation->x += item->mRotationSpeed->x * timeDiff;
			item->mRotation->z = 0.0f;
		}
		else if (item->mState == GameObject::States::BeingHijacked)
		{
			//rotation x
			item->mRotation->x = -30.0f;
			item->mRotation->z = 0.0f;
		}
	}
	else if (item->mType == GameObject::Types::HolyGrenade || item->mType == GameObject::Types::Tree)
	{
		if (item->mType == GameObject::Types::HolyGrenade && item->mState == GameObject::States::MovingToTree)
		{
			item->mRotation->x += item->mRotationSpeed->x * timeDiff;
		}
		else
		{
			if (item->mRotation->x <= (-item->mRotationMax->x)) { item->mRotationSpeed->x = abs(item->mRotationSpeed->x);		   }
			if (item->mRotation->x >= (+item->mRotationMax->x)) { item->mRotationSpeed->x = abs(item->mRotationSpeed->x) * -1.0f;  }
			item->mRotation->x += item->mRotationSpeed->x * timeDiff;

			if (item->mRotation->z <= -item->mRotationMax->z) { item->mRotationSpeed->z = abs(item->mRotationSpeed->z); }
			if (item->mRotation->z >= +item->mRotationMax->z) { item->mRotationSpeed->z = abs(item->mRotationSpeed->z) * -1.0f; }
			item->mRotation->z += item->mRotationSpeed->z * timeDiff;
			item->mRotation->y += item->mRotationSpeed->y * timeDiff;
		}
	}
	else if (item->mType == GameObject::Types::Moai)
	{
		mMoai->mRotation->y += mMoai->mRotationSpeed->y * timeDiff; 
	}
}

bool Sector::withinBorders(const glm::vec3 *shootPosition)
{
	if (this->mPosition->x + SECTOR_WIDTH_MINUS_MARGIN_PEOPLE/2 > shootPosition->x && this->mPosition->x - SECTOR_WIDTH_MINUS_MARGIN_PEOPLE/2 < shootPosition->x &&
		this->mPosition->z + SECTOR_LENGTH_MINUS_MARGIN_PEOPLE/2 > shootPosition->z && this->mPosition->z - SECTOR_LENGTH_MINUS_MARGIN_PEOPLE/2 < shootPosition->z) { return true; }
	return false;
}

float Sector::getVolume(SoundInterface::Sounds soundType)
{
	GameObject *item = NULL;
	float len = 100000, volume = 0.0f, lenMax = 500.0f;
	if (soundType == SoundInterface::Sounds::SoundHolyGrenadeCrafting &&  mHolyGrenade->mState == GameObject::States::Crafting &&  mCountCraftingPeople > 0)
	{
		len = glm::length(*mScene->mCamera->mPosition - *this->mHolyGrenade->mPosition);
		lenMax = 500.0f;
	}
	else if (soundType == SoundInterface::Sounds::SoundHolyGrenadeRolling && mHolyGrenade->mState == GameObject::States::MovingToTree)
	{
		len = glm::length(*mScene->mCamera->mPosition - *this->mHolyGrenade->mPosition);
		lenMax = 800.0f;
	}
	if (len <= lenMax)
	{
		volume = glm::clamp(1 - (len / lenMax), 0.0f, 1.0f );
	}
	return volume;
}

//void Sector::handleShoot(const int button, const glm::vec2 *mousePosition)
//{
//	double timeBefore = glfwGetTime();
//	for (int i=0;i<this->mPeopleGood->size();i++)  {  collisionWithShoot(this->mPeopleGood->at(i), button, mousePosition); }
//	for (int i=0;i<this->mPeopleBad->size();i++)  {  collisionWithShoot(this->mPeopleBad->at(i), button, mousePosition); }
//	double timeAfter = glfwGetTime();				
//	cout << "\r\nElapsed Time: (s) " << (timeAfter - timeBefore)*1000.0 << endl;
//}

//void Sector::collisionWithShoot(GameObject *item, const int button, const glm::vec2 *mousePosition)
//{
//	glm::vec4 pos = (*mScene->mProjectionMatrix) * (*mScene->mCamera->mViewMatrix) * item->getModelMatrix() * glm::vec4(0,1,0,1);
//	pos /= pos[3];
//	pos.y *= -1.0;
//	pos *= 0.5;
//	pos += 0.5;
//
//	pos.x *= mScene->mScreenWidth;
//	pos.y *= mScene->mScreenHeight;
//	pos.z = glm::length((*mScene->mCamera->mPosition) - (*item->mPosition)); 
//	*item->mPositionPixelSpace = glm::vec3(pos.x,pos.y,pos.z);
//	int tolerancex = (int)(mScene->mScreenWidth / (*item->mPositionPixelSpace).z * item->mScale->x);
//	int tolerancey = (int)(mScene->mScreenHeight / (*item->mPositionPixelSpace).z * item->mScale->y);
//
//	if ((glm::abs(mousePosition->x - (*item->mPositionPixelSpace).x) < tolerancex)	&& (glm::abs(mousePosition->y - (*item->mPositionPixelSpace).y) < tolerancey))
//	{
//
//	}
//}

void Sector::drawStoneWall(void)
{
	glActiveTexture(GL_TEXTURE0 + 1);	
	if (mBadState == -1) { glBindTexture(GL_TEXTURE_2D, mScene->mModelStoneWall->mTextureHandles[0]); }
	else { glBindTexture(GL_TEXTURE_2D, mScene->mModelStoneWall->mTextureHandles[1]); }
	mScene->setUniforms(mScene->mModelStoneWall,mStoneWall,this);
	mScene->drawModel(mScene->mModelStoneWall);
}

void Sector::drawMoai(void)
{
	if (this->mState != Sector::States::Won && this->mState != Sector::States::Lost)
	{
		if (mScene->checkFrustumCulling(this->mMoai))
		{		
			mScene->setUniforms(mScene->mModelMoai,mMoai, this);
			mScene->drawModel(mScene->mModelMoai);
		}
	}
}

void Sector::drawTree(bool bark)
{
	
	if (this->mTree->mState != GameObject::States::Dead && this->mState != Sector::States::Won && this->mState != Sector::States::Lost)
	{	
		if (mScene->checkFrustumCulling(this->mTree))
		{
			mScene->setUniforms(mScene->mModelTree,mTree, this);
			mScene->drawModel(mScene->mModelTree,"special", bark, 15);
		}
	}
}

void Sector::drawHolyGrenade(void)
{
	if (this->mHolyGrenade->mState != GameObject::States::Dead)
	{
		if (mScene->checkFrustumCulling(this->mHolyGrenade))
		{
			if (mHolyGrenade->mState == GameObject::States::CraftingFinished || mHolyGrenade->mState == GameObject::States::MovingToTree || mHolyGrenade->mState == GameObject::States::MovingToTreeFinished) { mScene->mModelHolyGrenade->mUseTexture = 5; } else { mScene->mModelHolyGrenade->mUseTexture = 1; } 
			mScene->setUniforms(mScene->mModelHolyGrenade,mHolyGrenade, this);
			mScene->drawModel(mScene->mModelHolyGrenade);
		}
	}
}

void Sector::drawHolyGrenadeSatellite(void)
{
	if (this->mHolyGrenade->mState == GameObject::States::CraftingFinished || this->mHolyGrenade->mState == GameObject::States::MovingToTree)
	{
		mScene->setUniforms(mScene->mModelHolyGrenade,mHolyGrenadeSatellite, this);
		mScene->drawModel(mScene->mModelHolyGrenade);
	}
}

void Sector::drawHolyGrenadeBorder(void)
{
	if (this->mHolyGrenadeBorder->mState != GameObject::States::Dead && this->mState != Sector::States::Won && this->mState != Sector::States::Lost)
	{
		mScene->setUniforms(mScene->mModelHolyGrenadeBorder,mHolyGrenadeBorder, this);
		mScene->drawModel(mScene->mModelHolyGrenade);
	}
}

void Sector::drawPeopleGood(void)
{
	for (int i=0;i<(int)mPeopleGood->size();i++)
	{
		if (mPeopleGood->at(i)->mState != GameObject::States::Dead)
		{
			if (mScene->checkFrustumCulling(mPeopleGood->at(i)))
			{			
				mScene->setUniforms(mScene->mModelPeopleGood,mPeopleGood->at(i), this);
				mScene->drawModel(mScene->mModelPeopleGood);
			}
			//else { cout << " gecullt good " << endl; }
		}
	}
	if (this->mState == Sector::States::Won) //draw wonloststatue as won
	{ 
		mScene->setUniforms(mScene->mModelPeopleGood,mWonLostStatue, this);
		mScene->drawModel(mScene->mModelPeopleGood);
	}
}

void Sector::drawPeopleBad(void)
{
	for (int i=0;i<(int)mPeopleBad->size();i++)
	{
		if (mPeopleBad->at(i)->mState != GameObject::States::Dead)
		{
			if (mScene->checkFrustumCulling(mPeopleBad->at(i)))
			{
				mScene->setUniforms(mScene->mModelPeopleBad,mPeopleBad->at(i), this);
				mScene->drawModel(mScene->mModelPeopleBad);
			}
			//else { cout << " gecullt bad " << endl; }
		}
	}
	if (this->mState == Sector::States::Lost) //draw wonloststatue as lost
	{ 
		mScene->setUniforms(mScene->mModelPeopleBad,mWonLostStatue, this);
		mScene->drawModel(mScene->mModelPeopleBad);
	}
}


