#ifndef _COMMON_HPP_
#define _COMMON_HPP_

#include "..\external\flextGL\flextGL.h"

#define GLFW_NO_GLU
#include <GL/glfw.h>

#include <iostream>
#include <string>

#include <glm/glm.hpp>
#include <glm/gtx/transform.hpp> 
#include <glm/gtc/type_ptr.hpp> 
#include <glm/gtx/random.hpp>
#include <glm/gtc/matrix_transform.hpp>

using std::string;
using std::cerr;
using std::cout;
using std::endl;

const string pathtofiles = "../TheVengefulHandOfGod/"; //for visual studio debugging
//const string pathtofiles = ""; //just for releases
const string pathtotextures = pathtofiles + "resources/textures/"; 
const string pathtomodels = pathtofiles + "resources/models/";
const string pathtosounds = pathtofiles + "resources/sounds/";
const string pathtoshaders = pathtofiles + "shaders/";


const int	SCREEN_WIDTH_DEFAULT = 1200;
const int	SCREEN_HEIGHT_DEFAULT = 800;
const int	SECTOR_COUNT = 5;
const int   SECTOR_WIDTH = 800; //default: 600
const int   SECTOR_LENGTH = 800; //default: 600
const int   SECTOR_WIDTH_MINUS_MARGIN_PEOPLE = (const int)(SECTOR_WIDTH / 1.3f); // default 1.3f
const int   SECTOR_LENGTH_MINUS_MARGIN_PEOPLE = (const int)(SECTOR_LENGTH / 1.3f); //default 1.3f
const int   SECTOR_WIDTH_MINUS_MARGIN_BIGOBJECTS = (const int)(SECTOR_WIDTH / 1.7f); //default 1.3f
const int   SECTOR_LENGTH_MINUS_MARGIN_BIGOBJECTS = (const int)(SECTOR_LENGTH / 1.7f); //default 1.3f

const int   GAMEFIELD_MAX = glm::max(SECTOR_WIDTH * (3 + 10),SECTOR_LENGTH * (3  + 10)); //should be sector_cols/rows instead of 3, but resize of mountains necessary
const int   GAMEFIELD_WIDTH = GAMEFIELD_MAX; //SECTOR_WIDTH * SECTOR_COLS * 2;
const int   GAMEFIELD_LENGTH = GAMEFIELD_MAX; //SECTOR_LENGTH * SECTOR_ROWS * 2;
const int	GAMEFIELD_LENGTH_MINUS_MARGIN_SECTORS = GAMEFIELD_MAX - SECTOR_LENGTH;
const int   FIELDSPERROW = 10; //default: 10

const float CAMERA_MOVE_PERSECOND = 250; // default 250
const float CAMERA_MOVE_FROMPARENT_PERSECOND = 1000; //default 360
const float CAMERA_ROTATEX_PERSECOND = 120; //default 120
const float CAMERA_ROTATEY_PERSECOND = 180; //default 80
const float PEOPLE_MOVE_PERSECOND = 40; //default: 20
const float LIGHTNING_MOVE_PERSECOND = 1.5f; //default: 1.5f
const float LIGHTNING_VISIBLE_SECONDS = 0.5f;
const float HOLYGRENADE_MOVE_PERSECOND = 10; //default: 10
const float CAMERA_HEIGHT_DEFAULT = GAMEFIELD_LENGTH / 3.0f; //default: 200
const float CAMERA_HEIGHT_MINIMUM = 10.0f; //default: 1
const float	MOAI_ROTATION_SPEED_MINIMUM = 25; //default: 25
const float	MOAI_ROTATION_SPEED_MAXIMUM = 50; //default: 50
const float TREE_ROTATION_SPEED_MINIMUM = 2; //default: 2
const float TREE_ROTATION_SPEED_MAXIMUM = 5; //default: 5
const float	HOLYGRENADE_ROTATION_SPEED = 50; //default: 100
const float GOODS_SPAWNTIME_MINIMUM_PERSECOND = 1.0f; //default: 1.0f
const float GOODS_SPAWNTIME_MAXIMUM_PERSECOND = 2.0f; //default: 2.0f
const int   PEOPLE_COUNT_PERSECTOR_MAXIMUM = 50; // default: 100
const float TREE_RADIUS_SUCKIN = 60; //default: 40
const float TREE_RADIUS_COLLISION = 10; //default: 10
const float HOLYGRENADE_RADIUS_SUCKIN = 70; //default: 40
const float HOLYGRENADE_RADIUS_COLLISION = 50; //default: 25
const int	HOLYGRENADE_CRAFTING_POINTSTOWIN = 4000; //default: 4000
const int	HOLYGRENADE_NOCRAFTING_POINTSTOLOSE = -1800; //default: -1800
const int	HOLYGRENADE_CRAFTING_POINTS_PERSECOND = 1; //default: 1
const int	HOLYGRENADE_NOCRAFTING_POINTS_PERSECOND = -10; //default: -1
const int   MOUSEWHEEL_SMOOTHNESS = 1; //default: 5
const float	PEOPLE_DYING_TIME = 1.0f; // default: 1.5f
//const float SPEEDFACTOR = 1.0f;

const GLfloat ground_positions[] = {  							
	//ground as 2 triangles
	-1.0f, 0.0f, +1.0f, +1.0f, 		//v0
	+1.0f, 0.0f, +1.0f, +1.0f,		//v1
	-1.0f, 0.0f, -1.0f, +1.0f,		//v2
											
	+1.0f, 0.0f, +1.0f, +1.0f,		//v1											
	+1.0f, 0.0f, -1.0f, +1.0f,		//v3
	-1.0f, 0.0f, -1.0f, +1.0f,		//v2
};

const GLfloat triangle_positions[] = {  											
	//just a few pyramid triangles as enemies
	-1.0f,  -0.0f, +1.0f, 1.0f, //v0
	+1.0f,  -0.0f, +1.0f, 1.0f, //v1
	+0.0f,  +2.0f, +0.0f, 1.0f, //v2

	+1.0f,  -0.0f, +1.0f, 1.0f, //v1
	+1.0f,  -0.0f, -1.0f, 1.0f, //v3
	+0.0f,  +2.0f, +0.0f, 1.0f, //v2
											
	+1.0f,  -0.0f, -1.0f, 1.0f, //v3
	-1.0f,  +0.0f, -1.0f, 1.0f, //v4
	+0.0f,  +2.0f, +0.0f, 1.0f, //v2											

	-1.0f,  +0.0f, -1.0f, 1.0f, //v4
	-1.0f,  -0.0f, +1.0f, 1.0f, //v0
	+0.0f,  +2.0f, +0.0f, 1.0f, //v2
};

const GLfloat skybox_positions[] = {
	//cube for skybox
	-1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v1
	-1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v0
	+1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v2

	+1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v2
	+1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v3
	-1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v1

	+1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v3
	+1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v2
	+1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v4

	+1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v4
	+1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v5
	+1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v3

	+1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v5
	+1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v4
	-1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v6

	-1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v6
	-1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v7
	+1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v5

	-1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v7
	-1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v6
	-1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v0

	-1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v0
	-1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v1
	-1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v7

	-1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v7
	-1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v1
	+1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v3

	+1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v3
	+1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v5
	-1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v7
		
	-1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v0
	-1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v6
	+1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v2

	+1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, +1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v2
	-1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v6
	+1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_WIDTH / 2.3f, -1.0f * GAMEFIELD_LENGTH / 2.3f, 1.0f, //v4
};


const GLfloat ground_texture_coordinates[] = {	
	0, 0, //v0
	1, 0, //v1
	0, 1, //v2

	1, 0, //v1
	1, 1, //v3
	0, 1, //v2
};
	
const GLfloat triangle_texture_coordinates[] = { //triangletexturecoordinates
	0, 0, //v0
	1, 0, //v1
	0.5f, 1, //v2

	0, 0, //v0
	1, 0, //v1
	0.5f, 1, //v2
											
	0, 0, //v0
	1, 0, //v1
	0.5f, 1, //v2											

	0, 0, //v0
	1, 0, //v1
	0.5f, 1, //v2
};

const GLfloat overlay_positions[] = {  							
	//ground as 2 triangles
	-1.0f, +1.0f, 0.0f, +1.0f, 		//v0
	+1.0f, +1.0f, 0.0f, +1.0f,		//v1
	-1.0f, -1.0f, 0.0f, +1.0f,		//v2
											
	+1.0f, +1.0f, 0.0f, +1.0f,		//v1											
	+1.0f, -1.0f, 0.0f, +1.0f,		//v3
	-1.0f, -1.0f, 0.0f, +1.0f,		//v2
};

const GLfloat overlay_texture_coordinates[] = {	
	0, 0, //v0
	1, 0, //v1
	0, 1, //v2

	1, 0, //v1
	1, 1, //v3
	0, 1, //v2
};


#endif //#ifndef _COMMON_HPP_
