#include "common.hpp"
#include "utility.hpp"
#include "Shader.hpp"
#include "Scene.hpp"
#include "SoundInterface.hpp"
#include "PickingHelper.hpp"

bool debugprint = false, fpsprint = false, FULLSCREEN = false, HIGHESTFRAMERATE = false;
Scene *scene;
													
PFNGLDELETEVERTEXARRAYSPROC my_glDeleteVertexArrays;

// Test if we got a valid forward compatible context (FCC)
void testOgl3(void)
{
	GLint profile;
	cout << "OpenGL Version: " << glGetString(GL_VERSION) << endl;
	glGetIntegerv(GL_CONTEXT_PROFILE_MASK, &profile);
	if (profile == GL_CONTEXT_CORE_PROFILE_BIT) { cout << "Having a core-profile" << endl; } 
	else { cout << "Having a compatibility-profile" << endl; }
	glGetError();
}

void resize(int x, int y)
{
	scene->resize(x,y);
}

void keycallback(int key, int state)
{
	
    //F1 - Help (if available)
    //F2 - Framerate on/off
    //F3 - Wire Frame on/off
    //F4 - Textur-Sampling-Quality: Nearest Neighbor/Bilinear
    //F5 - Mip Mapping-Quality: Off/Nearest Neighbor/Linear
    //F6 - -
    //F7 - -
    //F8 - Viewfrustum-Culling on/off
    //F9 - Transparency on/off

	if (state == GLFW_PRESS) 
	{
		if (key == GLFW_KEY_F1 && scene->mState != Scene::States::Init) {scene->togglePause(); }		
		if (key == GLFW_KEY_F2) {  fpsprint = !fpsprint; }			
		if (key == GLFW_KEY_F3) { scene->toggleWireFrame();}
		if (key == GLFW_KEY_F4) {scene->toggleTrilinear();}
		if (key == GLFW_KEY_F5) { scene->toggleMipMapping();}
		if (key == GLFW_KEY_F8) {scene->toggleFrustumCulling();}
		if (key == GLFW_KEY_F9) {scene->toggleTransparency();}
		if (key == GLFW_KEY_F10) {scene->mSoundInterface->toggleMute(); }			
		if (key == GLFW_KEY_F11) { scene->toggleNightMode(); } //toggle daymode/nightmode
		if (key == '1') { cout << "Use Linear Scaling = " << PickingHelper::getInstance()->toggleLinearScaling() << endl;}
		if (key == '2') { cout << "Use Naive Search = " << PickingHelper::getInstance()->toggleNaiveSearch() << endl;}
		if (key == '3') { cout << "Use Bubble Cursor = " << PickingHelper::getInstance()->toggleBubbleCursor() << endl;}
		if (key == '4') { PickingHelper::getInstance()->togglePickbufferRectangle();};
		if (key == '5') { cout << "Cutoff Distance = " << PickingHelper::getInstance()->changeCutoffDistance(-20) << endl;}
		if (key == '6') { cout << "Cutoff Distance = " << PickingHelper::getInstance()->changeCutoffDistance(20) << endl;}
		if (key == '7') { cout << "Minimum Scale = " << PickingHelper::getInstance()->changeMinScale(-0.1) << endl;}
		if (key == '8') { cout << "Minimum Scale = " << PickingHelper::getInstance()->changeMinScale(0.1) << endl;}
		if (key == '9') { cout << "Inverse Exponent = " << PickingHelper::getInstance()->changeInverseExponent(-0.5) << endl;}
		if (key == '0') { cout << "Inverse Exponent = " << PickingHelper::getInstance()->changeInverseExponent(0.5) << endl;}
		//if (key == 'R') {  scene->generateRandom();}
		//if (key == 'Z') {  debugprint = true; }
	}	
}

void mousebuttoncallback(int button, int action )
{
	if (action == GLFW_PRESS)
	{
		scene->handleShoot(button);
	}
}


int main(int argc, char* argv[]) 
{
	for(int i = 0; i < argc; i++)
	{
			cout << argv[i] << endl;
			if (strcmp(argv[i], "fullscreen") == 0) { FULLSCREEN = true; }
			if (strcmp(argv[i], "highestframerate") == 0) { HIGHESTFRAMERATE = true; }
	} 
	glfwInit();

	int screenwidth = SCREEN_WIDTH_DEFAULT;
	int screenheight = SCREEN_HEIGHT_DEFAULT;
	int windowmode = GLFW_WINDOW;
	
	if (FULLSCREEN)
	{
		RECT rc;
		GetWindowRect(GetDesktopWindow(), &rc);
		screenwidth = rc.right;
		screenheight = rc.bottom;
		windowmode = GLFW_FULLSCREEN;
		cout << rc.right << " " << rc.bottom << endl;	
	}

	// Set flags so GLFW creates the desired OpenGL context
	glfwOpenWindowHint(GLFW_OPENGL_VERSION_MAJOR, 3);
	glfwOpenWindowHint(GLFW_OPENGL_VERSION_MINOR, 3);
	glfwOpenWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

	if (glfwOpenWindow(	screenwidth, screenheight, 0,0,0,0, 24, 8, windowmode) != GL_TRUE) 
	{
			cerr << "Failed to initialize OpenGL window." << endl;
			glfwTerminate();
			return 1;
	}
	glfwEnable( GLFW_MOUSE_CURSOR );
	
	if (!flextInit()) // check for error 
	{ 
		cerr <<  "Error: initializing flextGL" << endl;
		return 1; 
	}
	glGetError(); 
	testOgl3();
	
	scene = new Scene();
	glfwSetWindowTitle("The Vengeful Hand Of God");
	glfwSetWindowSizeCallback(&resize);
	glfwSetKeyCallback(&keycallback);
	glfwSetMouseButtonCallback(&mousebuttoncallback);
	bool running = true;

	my_glDeleteVertexArrays = (PFNGLDELETEVERTEXARRAYSPROC)glfwGetProcAddress("glDeleteVertexArrays");
		
	double timeLast=0.0;
	double timeDiff=0.0;
	double timeNow=0.0;
	int countFrames = 0;
	double countMilliSeconds = 0.0;
	if (HIGHESTFRAMERATE) { glfwSwapInterval(0); }//set Framerate to Highest possible, otherwise its 60 fps

	while (running) 
	{
		timeNow = glfwGetTime();
		timeDiff=timeNow-timeLast;
		timeLast = timeNow;

		scene->update((float)(timeDiff) * scene->mSpeedFactor);
		scene->draw();
		glfwSwapBuffers();
		getErrors();

		countMilliSeconds+=timeDiff;
		countFrames++;

		if(countMilliSeconds>1.0) 
		{
			if (fpsprint == true) 
			{ 
				glfwSetWindowTitle(("Fps: " + string_tostring(countFrames)).c_str());
				//cout << "Fps: " << countFrames << endl; 
				//cout << "MsPerFrame: " << timeDiff << endl;
			}
			//if (scene->mFrustumCulling) { cout << "Drawn Objects: " << scene->mCountDrawnObjects / 2 << ", Culled Objects: " << scene->mCountCulledObjects / 2 << endl; }
			countMilliSeconds=0;
			countFrames=0;
		}
		// Check if the window has been closed
		running = running && !glfwGetKey( GLFW_KEY_ESC );
		running = running && glfwGetWindowParam( GLFW_OPENED );
	}
	//system("pause");
	delete(scene);
	glfwTerminate();
	return 0;
}
