#pragma once
#pragma warning( disable : 4482 4996 4800 )

#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <glm/gtx/transform.hpp> 
#include <assimp.hpp>
#include <aiScene.h>
#include <aiPostProcess.h>

#include "common.hpp"

/** @file */

//namespace glm {
//class mat3;
//}

/*struct sStdGeomVertex
{
	glm::vec4 pos;
	glm::vec3 normal;
	glm::vec2 tex;
	glm::vec3 tangent;
	glm::vec3 binormal;
};*/

class Shader;
class Model;

bool fileExists(const string &filename);
string readFile(const string &filename);
void getErrors(void);

void printMatrix(glm::mat3 matrix);
void printMatrix(glm::mat4 matrix);
void printVector(glm::vec2 vec);
void printVector(glm::vec3 vec);
void printVector(glm::vec4 vec);
void printArray(float *printarray, int arraysize);
float randFloat(float a, float b);
int randInt(int a, int b);
int sign(float a);
GLbyte *gltReadTGABits(const char *szFileName, GLint *iWidth, GLint *iHeight, GLint *iComponents, GLenum *eFormat);
GLuint load2dTexture(const string& path, bool projection = false);
float *calculateNormals(const float *vertices_in, int arraysize);

void setupCubeMap(GLuint& texture);
GLuint loadCubeMap(string paths[]);
void unloadCubeMap(GLuint& texture); 
void initVboVao(const Shader& shader, GLuint *vbo_id, GLuint *vao_id, const GLfloat *positions, GLfloat *normals, const GLfloat *texcoordinates, int sizeofarray, int buffermode, bool update);
void initVboVaoFromModel(const Shader &shader, Model *model, const string& path, int buffermode);
void releaseVboVao(GLuint *vbo_id, GLuint *vao_id);

	/**
	* Initializes the FBO, which is used as PickBuffer  (render objectIds for object picking).
	* @param fboId is used to store the handle the FBO has got
	* @param imgId is used to store the texture handle
	* @param width determines the width of the FBO
	* @param height determines the height of the FBO
	* @return if the FBO could be created successful
	*/
bool initFbo(GLuint &fboId, GLuint &imgId, int width, int height);
int  readPixel(unsigned int x, unsigned int y, GLuint fboId, GLuint imgId);

	/**
	* Reads pixels from the GPU in a rectangular form.
	* @param xFrom Left start value of the rectangle
	* @param yFrom Bottom start value of the rectangle
	* @param width Width of the rectangle
	* @param height Height of the rectangle
	* @param fboId is used to identify the FBO
	* @param imgId is used to identify the texture
	* @return the read values as integer array 
	*/
int *readPixels(unsigned int xFrom, unsigned int yFrom, unsigned int width, unsigned int height, GLuint fboId, GLuint imgId);
template <class T> string string_tostring(const T& t, const int digits = 1) { std::stringstream ss; ss << std::setw(digits) << std::setfill('0') << t;  return ss.str(); }
